#!/usr/bin/env python3
'''
GUI for adding new links to MAVProxy
'''

import wx
from pymavlink import mavutil

from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import multiproc
import MAVProxy.modules.mavproxy_link

class linkAddDialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        super(linkAddDialog, self).__init__(*args, **kwds)
    
        self.panelGUI = wx.Panel(self, wx.ID_ANY)

        self.sizerGUI = wx.FlexGridSizer(5, 2, 0, 0)
        
        self.addLink = None
        
        self.conStr = None

        label_1 = wx.StaticText(self.panelGUI, wx.ID_ANY, "Connection Type:")
        self.sizerGUI.Add(label_1, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL | wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 3)

        self.choiceConnection = wx.Choice(self.panelGUI, wx.ID_ANY, choices=["udpin", "udpout", "tcpin", "tcp", "Serial"])
        self.choiceConnection.SetSelection(0)
        self.sizerGUI.Add(self.choiceConnection, 0, wx.ALL, 3)

        self.labelConType = wx.StaticText(self.panelGUI, wx.ID_ANY, "IP:Port")
        self.sizerGUI.Add(self.labelConType, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL, 3)

        self.textConIPPort = wx.TextCtrl(self.panelGUI, wx.ID_ANY, "127.0.0.1:14550")
        self.textConIPPort.SetMinSize((150, 34))
        self.sizerGUI.Add(self.textConIPPort, 0, wx.ALL | wx.EXPAND, 3)

        self.labelSerialPort = wx.StaticText(self.panelGUI, wx.ID_ANY, "Port:")
        self.sizerGUI.Add(self.labelSerialPort, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL | wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 3)

        self.choiceSerialPort = wx.Choice(self.panelGUI, wx.ID_ANY, choices=[])
        
        self.sizerGUI.Add(self.choiceSerialPort, 0, wx.ALL | wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 3)

        self.labelBaud = wx.StaticText(self.panelGUI, wx.ID_ANY, "Baud Rate:")
        self.sizerGUI.Add(self.labelBaud, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALL | wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 3)

        self.choiceBaud = wx.Choice(self.panelGUI, wx.ID_ANY, choices=["9600", "19200", "38400", "57600", "115200", "921600", "1500000"])
        self.choiceBaud.SetSelection(3)
        self.sizerGUI.Add(self.choiceBaud, 0, wx.ALL | wx.RESERVE_SPACE_EVEN_IF_HIDDEN, 3)

        self.buttonAdd = wx.Button(self.panelGUI, wx.ID_ADD, "Add Link")
        self.sizerGUI.Add(self.buttonAdd, 0, wx.ALL, 3)

        self.buttonExit = wx.Button(self.panelGUI, wx.ID_CANCEL, "")
        self.sizerGUI.Add(self.buttonExit, 0, wx.ALIGN_CENTER_VERTICAL, 0)

        self.panelGUI.SetSizer(self.sizerGUI)

        self.Layout()

        self.Bind(wx.EVT_CHOICE, self.onChangeType, self.choiceConnection)
        self.Bind(wx.EVT_BUTTON, self.onAdd, self.buttonAdd)
        self.Bind(wx.EVT_BUTTON, self.onClose, self.buttonExit)
        self.Bind(wx.EVT_CLOSE, self.onClose)
        
        # Set initial state
        self.choiceSerialPort.Disable()
        self.choiceBaud.Disable()
        self.labelSerialPort.Disable()
        self.labelBaud.Disable()
        self.textConIPPort.Enable()
        self.labelConType.Enable()
        
        ports = mavutil.auto_detect_serial(preferred_list=MAVProxy.modules.mavproxy_link.preferred_ports)
        for p in ports:
            self.choiceSerialPort.Append(p.device)
        self.choiceSerialPort.SetSelection(0)

    def onChangeType(self, event):
        ''' Change between network and serial connection options '''
        choice = self.choiceConnection.GetString( self.choiceConnection.GetSelection())
        if choice in ["udpin", "udpout", "tcpin", "tcp"]:
            self.choiceSerialPort.Disable()
            self.choiceBaud.Disable()
            self.labelSerialPort.Disable()
            self.labelBaud.Disable()
            self.textConIPPort.Enable()
            self.labelConType.Enable()
        else:
            self.choiceSerialPort.Enable()
            self.choiceBaud.Enable()
            self.labelSerialPort.Enable()
            self.labelBaud.Enable()
            self.textConIPPort.Disable()
            self.labelConType.Disable()
            
            self.choiceSerialPort.Clear()
            ports = mavutil.auto_detect_serial(preferred_list=MAVProxy.modules.mavproxy_link.preferred_ports)
            for p in ports:
                self.choiceSerialPort.Append(p.device)
            self.choiceSerialPort.SetSelection(0)
        
    def onAdd(self, event):
        '''Return connection string'''
        choice = self.choiceConnection.GetString( self.choiceConnection.GetSelection())
        self.conStr = None
        if choice in ["udpin", "udpout", "tcpin", "tcp"]:
            self.conStr = "" + choice + ":" + self.textConIPPort.GetValue()
        else:
            self.conStr = "" + self.choiceSerialPort.GetString(self.choiceSerialPort.GetSelection()) + ":" + self.choiceBaud.GetString(self.choiceBaud.GetSelection())
        #print("1. " + self.conStr)
        self.EndModal(wx.ID_ADD)
        
    def onClose(self, event):
        ''' Exit the dialog (cancel) '''
        if event is None and self.IsModal():
            self.EndModal(wx.ID_ADD)
        elif self.IsModal():
            self.EndModal(event.EventObject.Id)
        else:
            self.Close()

class MPMenulinkAddDialog(object):
    '''used to create a file dialog callback'''
    def __init__(self):
        pass

    def call(self):
        '''show a file dialog'''
        from MAVProxy.modules.lib.wx_loader import wx

        dlg = linkAddDialog(None, title='Add New Link')
        if dlg.ShowModal() != wx.ID_ADD:
            return None
        else:
            # get the connection string before closing dialog
            Constr = dlg.conStr
            dlg.Destroy()
            return Constr

if __name__ == "__main__":
    app = wx.App(False)
    
    dlg = linkAddDialog(None, title='Add New Link')
    if dlg.ShowModal() != wx.ID_ADD:
        print("cancelled")
        dlg.Destroy()
    else:
        # get the connection string before closing dialog
        Constr = dlg.conStr
        print("OK: " + Constr)
        dlg.Destroy()
            
    app.MainLoop()
    



#!/usr/bin/env python

# Lat Long - UTM, UTM - Lat Long conversions
#
# see http://www.pygps.org
#

from math import pi, sin, cos, tan, sqrt

#LatLong- UTM conversion..h
#definitions for lat/long to UTM and UTM to lat/lng conversions
#include <string.h>

_deg2rad = pi / 180.0
_rad2deg = 180.0 / pi

_EquatorialRadius = 2
_eccentricitySquared = 3

_ellipsoid = [
#  id, Ellipsoid name, Equatorial Radius, square of eccentricity
# first once is a placeholder only, To allow array indices to match id numbers
	[ -1, "Placeholder", 0, 0],
	[ 1, "Airy", 6377563, 0.00667054],
	[ 2, "Australian National", 6378160, 0.006694542],
	[ 3, "Bessel 1841", 6377397, 0.006674372],
	[ 4, "Bessel 1841 (Nambia] ", 6377484, 0.006674372],
	[ 5, "Clarke 1866", 6378206, 0.006768658],
	[ 6, "Clarke 1880", 6378249, 0.006803511],
	[ 7, "Everest", 6377276, 0.006637847],
	[ 8, "Fischer 1960 (Mercury] ", 6378166, 0.006693422],
	[ 9, "Fischer 1968", 6378150, 0.006693422],
	[ 10, "GRS 1967", 6378160, 0.006694605],
	[ 11, "GRS 1980", 6378137, 0.00669438],
	[ 12, "Helmert 1906", 6378200, 0.006693422],
	[ 13, "Hough", 6378270, 0.00672267],
	[ 14, "International", 6378388, 0.00672267],
	[ 15, "Krassovsky", 6378245, 0.006693422],
	[ 16, "Modified Airy", 6377340, 0.00667054],
	[ 17, "Modified Everest", 6377304, 0.006637847],
	[ 18, "Modified Fischer 1960", 6378155, 0.006693422],
	[ 19, "South American 1969", 6378160, 0.006694542],
	[ 20, "WGS 60", 6378165, 0.006693422],
	[ 21, "WGS 66", 6378145, 0.006694542],
	[ 22, "WGS-72", 6378135, 0.006694318],
	[ 23, "WGS-84", 6378137, 0.00669438]
]

#Reference ellipsoids derived from Peter H. Dana's website-
#http://www.utexas.edu/depts/grg/gcraft/notes/datum/elist.html
#Department of Geography, University of Texas at Austin
#Internet: pdana@mail.utexas.edu
#3/22/95

#Source
#Defense Mapping Agency. 1987b. DMA Technical Report: Supplement to Department of Defense World Geodetic System
#1984 Technical Report. Part I and II. Washington, DC: Defense Mapping Agency

#def LLtoUTM(int ReferenceEllipsoid, const double Lat, const double Long,
#			 double &UTMNorthing, double &UTMEasting, char* UTMZone)

def LLtoUTM( Lat, Long, ReferenceEllipsoid=23):
    """
    converts lat/long to UTM coords.  Equations from USGS Bulletin 1532
    East Longitudes are positive, West longitudes are negative.
    North latitudes are positive, South latitudes are negative
    Lat and Long are in decimal degrees
    Written by Chuck Gantz- chuck.gantz@globalstar.com
    """
    a = _ellipsoid[ReferenceEllipsoid][_EquatorialRadius]
    eccSquared = _ellipsoid[ReferenceEllipsoid][_eccentricitySquared]
    k0 = 0.9996

#Make sure the longitude is between -180.00 .. 179.9
    LongTemp = (Long+180)-int((Long+180)/360)*360-180 # -180.00 .. 179.9

    LatRad = Lat*_deg2rad
    LongRad = LongTemp*_deg2rad

    ZoneNumber = int((LongTemp + 180)/6) + 1

    if Lat >= 56.0 and Lat < 64.0 and LongTemp >= 3.0 and LongTemp < 12.0:
        ZoneNumber = 32

    # Special zones for Svalbard
    if Lat >= 72.0 and Lat < 84.0:
        if  LongTemp >= 0.0  and LongTemp <  9.0:ZoneNumber = 31
        elif LongTemp >= 9.0  and LongTemp < 21.0: ZoneNumber = 33
        elif LongTemp >= 21.0 and LongTemp < 33.0: ZoneNumber = 35
        elif LongTemp >= 33.0 and LongTemp < 42.0: ZoneNumber = 37

    LongOrigin = (ZoneNumber - 1)*6 - 180 + 3 #+3 puts origin in middle of zone
    LongOriginRad = LongOrigin * _deg2rad

    #compute the UTM Zone from the latitude and longitude
    UTMZone = "%d%c" % (ZoneNumber, _UTMLetterDesignator(Lat))

    eccPrimeSquared = (eccSquared)/(1-eccSquared)
    N = a/sqrt(1-eccSquared*sin(LatRad)*sin(LatRad))
    T = tan(LatRad)*tan(LatRad)
    C = eccPrimeSquared*cos(LatRad)*cos(LatRad)
    A = cos(LatRad)*(LongRad-LongOriginRad)

    M = a*((1
            - eccSquared/4
            - 3*eccSquared*eccSquared/64
            - 5*eccSquared*eccSquared*eccSquared/256)*LatRad
           - (3*eccSquared/8
              + 3*eccSquared*eccSquared/32
              + 45*eccSquared*eccSquared*eccSquared/1024)*sin(2*LatRad)
           + (15*eccSquared*eccSquared/256 + 45*eccSquared*eccSquared*eccSquared/1024)*sin(4*LatRad)
           - (35*eccSquared*eccSquared*eccSquared/3072)*sin(6*LatRad))

    UTMEasting = (k0*N*(A+(1-T+C)*A*A*A/6
                        + (5-18*T+T*T+72*C-58*eccPrimeSquared)*A*A*A*A*A/120)
                  + 500000.0)

    UTMNorthing = (k0*(M+N*tan(LatRad)*(A*A/2+(5-T+9*C+4*C*C)*A*A*A*A/24
                                        + (61
                                           -58*T
                                           +T*T
                                           +600*C
                                           -330*eccPrimeSquared)*A*A*A*A*A*A/720)))

    if Lat < 0:
        UTMNorthing = UTMNorthing + 10000000.0; #10000000 meter offset for southern hemisphere
    #UTMZone was originally returned here.  I don't know what the
    #letter at the end was for.
    #print("UTMZone", UTMZone)
    return (ZoneNumber, UTMEasting, UTMNorthing)


def _UTMLetterDesignator(Lat):
#This routine determines the correct UTM letter designator for the given latitude
#returns 'Z' if latitude is outside the UTM limits of 84N to 80S
#Written by Chuck Gantz- chuck.gantz@globalstar.com

    if 84 >= Lat >= 72: return 'X'
    elif 72 > Lat >= 64: return 'W'
    elif 64 > Lat >= 56: return 'V'
    elif 56 > Lat >= 48: return 'U'
    elif 48 > Lat >= 40: return 'T'
    elif 40 > Lat >= 32: return 'S'
    elif 32 > Lat >= 24: return 'R'
    elif 24 > Lat >= 16: return 'Q'
    elif 16 > Lat >= 8: return 'P'
    elif  8 > Lat >= 0: return 'N'
    elif  0 > Lat >= -8: return 'M'
    elif -8> Lat >= -16: return 'L'
    elif -16 > Lat >= -24: return 'K'
    elif -24 > Lat >= -32: return 'J'
    elif -32 > Lat >= -40: return 'H'
    elif -40 > Lat >= -48: return 'G'
    elif -48 > Lat >= -56: return 'F'
    elif -56 > Lat >= -64: return 'E'
    elif -64 > Lat >= -72: return 'D'
    elif -72 > Lat >= -80: return 'C'
    else: return 'Z'	# if the Latitude is outside the UTM limits

#void UTMtoLL(int ReferenceEllipsoid, const double UTMNorthing, const double UTMEasting, const char* UTMZone,
#			  double& Lat,  double& Long )

def UTMtoLL(northing, easting, zone, isSouthernHemisphere=True,
            ReferenceEllipsoid=23):
    """
    converts UTM coords to lat/long.  Equations from USGS Bulletin 1532
    East Longitudes are positive, West longitudes are negative.
    North latitudes are positive, South latitudes are negative
    Lat and Long are in decimal degrees.
    Written by Chuck Gantz- chuck.gantz@globalstar.com
    Converted to Python by Russ Nelson <nelson@crynwr.com>

    FIXME: This is set up to work for the Southern Hemisphere.

Using
http://www.ga.gov.au/geodesy/datums/redfearn_geo_to_grid.jsp

    Site Name:    GDA-MGA: (UTM with GRS80 ellipsoid)
Zone:   36
Easting:  511669.521  Northing: 19328195.112
Latitude:   84  0 ' 0.00000 ''  Longitude: 34  0 ' 0.00000 ''
Grid Convergence:  0  -59 ' 40.28 ''  Point Scale: 0.99960166

____________
Site Name:    GDA-MGA: (UTM with GRS80 ellipsoid)
Zone:   36
Easting:  519384.803  Northing: 1118247.585
Latitude:   -80  0 ' 0.00000 ''  Longitude: 34  0 ' 0.00000 ''
Grid Convergence:  0  59 ' 5.32 ''  Point Scale: 0.99960459
___________
Site Name:    GDA-MGA: (UTM with GRS80 ellipsoid)
Zone:   36
Easting:  611263.812  Northing: 10110547.106
Latitude:   1  0 ' 0.00000 ''  Longitude: 34  0 ' 0.00000 ''
Grid Convergence:  0  -1 ' 2.84 ''  Point Scale: 0.99975325
______________
Site Name:    GDA-MGA: (UTM with GRS80 ellipsoid)
Zone:   36
Easting:  611263.812  Northing: 9889452.894
Latitude:   -1  0 ' 0.00000 ''  Longitude: 34  0 ' 0.00000 ''
Grid Convergence:  0  1 ' 2.84 ''  Point Scale: 0.99975325

So this uses a false northing of 10000000 in the both hemispheres.
ArcGIS used a false northing of 0 in the northern hem though.
Therefore it is difficult to actually know what hemisphere you are in.
    """
    k0 = 0.9996
    a = _ellipsoid[ReferenceEllipsoid][_EquatorialRadius]
    eccSquared = _ellipsoid[ReferenceEllipsoid][_eccentricitySquared]
    e1 = (1-sqrt(1-eccSquared))/(1+sqrt(1-eccSquared))

    x = easting - 500000.0 #remove 500,000 meter offset for longitude
    y = northing

    ZoneNumber = int(zone)
    if isSouthernHemisphere:
        y -= 10000000.0         # remove 10,000,000 meter offset used
                                # for southern hemisphere

    LongOrigin = (ZoneNumber - 1)*6 - 180 + 3  # +3 puts origin in middle of zone

    eccPrimeSquared = (eccSquared)/(1-eccSquared)

    M = y / k0
    mu = M/(a*(1-eccSquared/4-3*eccSquared*eccSquared/64-5*eccSquared*eccSquared*eccSquared/256))

    phi1Rad = (mu + (3*e1/2-27*e1*e1*e1/32)*sin(2*mu)
               + (21*e1*e1/16-55*e1*e1*e1*e1/32)*sin(4*mu)
               +(151*e1*e1*e1/96)*sin(6*mu))
    phi1 = phi1Rad*_rad2deg;

    N1 = a/sqrt(1-eccSquared*sin(phi1Rad)*sin(phi1Rad))
    T1 = tan(phi1Rad)*tan(phi1Rad)
    C1 = eccPrimeSquared*cos(phi1Rad)*cos(phi1Rad)
    R1 = a*(1-eccSquared)/pow(1-eccSquared*sin(phi1Rad)*sin(phi1Rad), 1.5)
    D = x/(N1*k0)

    Lat = phi1Rad - (N1*tan(phi1Rad)/R1)*(D*D/2-(5+3*T1+10*C1-4*C1*C1-9*eccPrimeSquared)*D*D*D*D/24
                                          +(61+90*T1+298*C1+45*T1*T1-252*eccPrimeSquared-3*C1*C1)*D*D*D*D*D*D/720)
    Lat = Lat * _rad2deg

    Long = (D-(1+2*T1+C1)*D*D*D/6+(5-2*C1+28*T1-3*C1*C1+8*eccPrimeSquared+24*T1*T1)
            *D*D*D*D*D/120)/cos(phi1Rad)
    Long = LongOrigin + Long * _rad2deg
    return (Lat, Long)

if __name__ == '__main__':
    (z, e, n) = LLtoUTM(45.00, -75.00, 23)
    print(z, e, n)
    (lat, lon) = UTMtoLL(n, e, z, 23)
    print(lat, lon)



#!/usr/bin/env python
'''
Generator Module - module for generators reporting via the GENERATOR_STATUS message
Peter Barker, Jun 2020
'''

import os
import os.path
import sys
from pymavlink import mavutil
import errno
import time

from MAVProxy.modules.lib import mp_module
from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import mp_settings


class generator(mp_module.MPModule):
    def __init__(self, mpstate):
        """Initialise module"""
        super(generator, self).__init__(mpstate, "generator", "")

        self.generator_settings = mp_settings.MPSettings(
            [ ('verbose', bool, False),
          ])
        self.add_command('generator',
                         self.cmd_generator,
                         "generator module",
                         ['status','set (LOGSETTING)'])
        self.console_row = 6
        self.console.set_status('Generator', 'No generator messages', row=self.console_row)
        self.last_seen_generator_message = 0
        self.mpstate = mpstate
        self.last_set_interval_sent = 0

    def usage(self):
        '''show help on command line options'''
        return "Usage: generator <status|set>"

    def cmd_generator(self, args):
        '''control behaviour of the module'''
        if len(args) == 0:
            print(self.usage())
        elif args[0] == "status":
            print(self.status())
        elif args[0] == "set":
            self.generator_settings.command(args[1:])
        else:
            print(self.usage())

    def mavlink_packet(self, m):
        '''handle mavlink packets'''
        if m.get_type() == 'GENERATOR_STATUS':
            if self.generator_settings.verbose:
                print("Got generator message")
            self.last_seen_generator_message = time.time()
            error_string = ""
            errors = []
            prefix = "MAV_GENERATOR_STATUS_FLAG_"
            len_prefix = len(prefix)
            flags = []
            for i in range(64):
                if m.status & (1<<i):
                    try:
                        name = mavutil.mavlink.enums["MAV_GENERATOR_STATUS_FLAG"][1<<i].name
                        if name.startswith(prefix):
                            name = name[len_prefix:]
                    except KeyError:
                        name = "UNKNOWN=%u" % (1<<i)
                    flags.append(name)
            self.console.set_status(
                'Generator',
                'Generator: RPM:%u current:%u volts:%f flags:%s runtime:%u maint:%d' %
                (m.generator_speed,
                 m.load_current,
                 m.bus_voltage,
                 ",".join(flags),
                 m.runtime,
                 m.time_until_maintenance,
                 ),
                row=self.console_row)

        now  = time.time()
        if now - self.last_seen_generator_message > 10:
            # request the message once per second:
            if now - self.last_set_interval_sent > 1:
                self.last_set_interval_sent = now
                self.master.mav.command_long_send(
                    self.mpstate.settings.target_system,
                    self.mpstate.settings.target_component,
                    mavutil.mavlink.MAV_CMD_SET_MESSAGE_INTERVAL,
                    0,  # confirmation
                    mavutil.mavlink.MAVLINK_MSG_ID_GENERATOR_STATUS,  # msg id
                    100000,  # interval
                    0,  # p3
                    0,  # p4
                    0,  # p5
                    0,  # p6
                    0)  # p7


def init(mpstate):
    '''initialise module'''
    return generator(mpstate)



#!/usr/bin/env python
'''
Event class and enums for Mission Editor
Michael Day
June 2014
'''
#MissionEditorEvents come FROM the GUI (with a few exceptions where the Mission Editor Module sends a message to itself, e.g., MEE_TIME_TO_QUIT)
#MissionEditorGUIEvents go TO the GUI
#enum for MissionEditorEvent types
MEE_READ_WPS = 0
MEE_WRITE_WPS = 1
MEE_TIME_TO_QUIT = 2
MEE_GET_WP_RAD = 3
MEE_GET_LOIT_RAD = 4
MEE_GET_WP_DEFAULT_ALT = 5
MEE_WRITE_WP_NUM = 6
MEE_LOAD_WP_FILE = 7
MEE_SAVE_WP_FILE = 8
MEE_SET_WP_RAD = 9
MEE_SET_LOIT_RAD = 10
MEE_SET_WP_DEFAULT_ALT = 11
#enum of MissionEditorGUIEvent types
MEGE_CLEAR_MISS_TABLE = 0
MEGE_ADD_MISS_TABLE_ROWS = 1
MEGE_SET_MISS_ITEM = 2
MEGE_SET_WP_RAD = 3
MEGE_SET_LOIT_RAD = 4
MEGE_SET_WP_DEFAULT_ALT = 5
MEGE_SET_LAST_MAP_CLICK_POS = 6

class MissionEditorEvent:
    def __init__(self, type, **kwargs):
        self.type = type
        self.arg_dict = kwargs

        if not self.type in [MEE_READ_WPS, MEE_WRITE_WPS, MEGE_CLEAR_MISS_TABLE,
                             MEGE_ADD_MISS_TABLE_ROWS, MEGE_SET_MISS_ITEM, MEE_TIME_TO_QUIT,
                             MEE_GET_WP_RAD, MEE_GET_LOIT_RAD, MEGE_SET_WP_RAD, MEGE_SET_LOIT_RAD,
                             MEE_GET_WP_DEFAULT_ALT, MEGE_SET_WP_DEFAULT_ALT, MEE_WRITE_WP_NUM,
                             MEE_LOAD_WP_FILE, MEE_SAVE_WP_FILE, MEE_SET_WP_RAD, MEE_SET_LOIT_RAD,
                             MEE_SET_WP_DEFAULT_ALT]:
            raise TypeError("Unrecongized MissionEditorEvent type:" + str(self.type))

    def get_type(self):
        return self.type

    def get_arg(self, key):
        if not key in self.arg_dict:
            print("No key %s in %s" % (key, str(self.type)))
            return None
        return self.arg_dict[key]



#!/usr/bin/env python

'''This is a script that will help you identify controls on your joystick
in order to create an appropriate joystick definition for the joystick
module.  You can run it like this:

    python -m MAVProxy.modules.mavproxy_joystick.findjoy
'''

from __future__ import print_function

import argparse
import collections
import pygame
import sys
import time
import time

State = collections.namedtuple('State',
                               ['axes', 'buttons', 'hats', 'balls'])


def parse_args():
    p = argparse.ArgumentParser()
    p.add_argument('joystick', nargs='?', type=int)

    return p.parse_args()


def list_joysticks():
    '''Print a list of available joysticks'''

    print('Available joysticks:')
    print()
    for jid in range(pygame.joystick.get_count()):
        j = pygame.joystick.Joystick(jid)
        print('({}) {}'.format(jid, j.get_name()))


def select_joystick():
    '''Allow user to select a joystick from a menu'''

    list_joysticks()

    while True:
        print('Select a joystick (L to list, Q to quit)'),
        choice = sys.stdin.readline().strip()

        if choice.lower() == 'l':
            list_joysticks()
        elif choice.lower() == 'q':
            return
        elif choice.isdigit():
            jid = int(choice)
            if jid not in range(pygame.joystick.get_count()):
                print('Invalid joystick.')
                continue
            break
        else:
            print('What?')

    return jid


def get_joystick_state(joy):
    axes = [joy.get_axis(x) for x in range(joy.get_numaxes())]
    buttons = [joy.get_button(x) for x in range(joy.get_numbuttons())]
    hats = [joy.get_hat(x) for x in range(joy.get_numhats())]
    balls = [joy.get_ball(x) for x in range(joy.get_numballs())]

    state = State(axes, buttons, hats, balls)

    return state


def main():
    print('\n'.join([
        'This script will help you identify axis, button, hat, and ',
        'ball ids on your joystick in order to create new joystick ',
        'definitions for the mavproxy_joystick module.',
        ''
    ]))

    pygame.init()
    pygame.joystick.init()

    args = parse_args()

    if args.joystick is None:
        args.joystick = select_joystick()
        if args.joystick is None:
            print('No joystick selected.')
            return

    joy = pygame.joystick.Joystick(args.joystick)
    joy.init()

    print('Joystick: {}'.format(joy.get_name()))
    print('  Axes: {}'.format(joy.get_numaxes()))
    print('  Buttons: {}'.format(joy.get_numbuttons()))
    print('  Hats: {}'.format(joy.get_numhats()))
    print('  Balls: {}'.format(joy.get_numballs()))
    print()

    pygame.event.clear()
    while True:
        print('Move a control on your joystick')
        print()
        e = pygame.event.wait()
        if e.type == pygame.JOYAXISMOTION:
            print('Axis', e.axis)
        elif e.type in [pygame.JOYBUTTONDOWN, pygame.JOYBUTTONUP]:
            print('Button', e.button)
        elif e.type == pygame.JOYHATMOTION:
            print('Hat', e.hat)
        elif e.type == pygame.JOYBALLMOTION:
            print('Ball', e.ball)

        # consume events
        t0 = time.time()
        while True:
            t1 = time.time()
            if not pygame.event.get() and (t1 - t0 > 1):
                break

if __name__ == '__main__':
    main()



"""
sc_webcam.py

This file includes functions to:
    initialise a web cam
    capture image from web cam

Image size is held in the smart_camera.cnf
"""

import sys
import time
import math
import cv2
import sc_config

class SmartCameraWebCam:

    def __init__(self, instance):

        # health
        self.healthy = False;

        # record instance
        self.instance = instance
        self.config_group = "camera%d" % self.instance

        # get image resolution
        self.img_width = sc_config.config.get_integer(self.config_group,'width',640)
        self.img_height = sc_config.config.get_integer(self.config_group,'height',480)

        # background image processing variables
        self.img_counter = 0        # num images requested so far

        # latest image captured
        self.latest_image = None

        # setup video capture
        self.camera = cv2.VideoCapture(self.instance)

        # check we can connect to camera
        if not self.camera.isOpened():
            print("failed to open webcam %d" % self.instance)

    # __str__ - print position vector as string
    def __str__(self):
        return "SmartCameraWebCam Object W:%d H:%d" % (self.img_width, self.img_height)

    # latest_image - returns latest image captured
    def get_latest_image(self):
        # write to file
        #imgfilename = "C:\Users\rmackay9\Documents\GitHub\ardupilot-balloon-finder\smart_camera\img%d-%d.jpg" % (cam_num,cam.get_image_counter())
        imgfilename = "img%d-%d.jpg" % (self.instance,self.get_image_counter())
        print (imgfilename)
        cv2.imwrite(imgfilename, self.latest_image)
        return self.latest_image

    # get_image_counter - returns number of images captured since startup
    def get_image_counter(self):
        return self.img_counter

    # take_picture - take a picture
    #   returns True on success
    def take_picture(self):
        # setup video capture
        print("Taking Picture")
        self.camera = cv2.VideoCapture(self.instance)
        self.camera.set(cv2.cv.CV_CAP_PROP_FRAME_WIDTH,self.img_width)
        self.camera.set(cv2.cv.CV_CAP_PROP_FRAME_HEIGHT,self.img_height)

        # check we can connect to camera
        if not self.camera.isOpened():
            self.healty = False
            return False

        # get an image from the webcam
        success_flag, self.latest_image=self.camera.read()

        # release camera
        self.camera.release()

        # if successful overwrite our latest image
        if success_flag:
            self.img_counter = self.img_counter+1
            return True

        # return failure
        return False

    # main - tests SmartCameraWebCam class
    def main(self):

        while True:
            # send request to image capture for image
            if self.take_picture():
                # display image
                cv2.imshow ('image_display', self.get_latest_image())
            else:
                print("no image")

            # check for ESC key being pressed
            k = cv2.waitKey(5) & 0xFF
            if k == 27:
                break

            # take a rest for a bit
            time.sleep(0.01)

# run test run from the command line
if __name__ == "__main__":
    sc_webcam0 = SmartCameraWebCam(0)
    sc_webcam0.main()



from MAVProxy.modules.lib.wx_loader import wx
from MAVProxy.modules.lib import icon
import time
import numpy, pylab

class GraphFrame(wx.Frame):
    """ The main frame of the application
    """

    def __init__(self, state):
        wx.Frame.__init__(self, None, -1, state.title)
        try:
            self.SetIcon(icon.SimpleIcon().get_ico())
        except Exception:
            pass
        self.state = state
        self.data = []
        for i in range(len(state.fields)):
            self.data.append([])
        self.paused = False

        self.create_main_panel()

        self.Bind(wx.EVT_IDLE, self.on_idle)

        self.redraw_timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.on_redraw_timer, self.redraw_timer)
        self.redraw_timer.Start(int(1000*self.state.tickresolution))

        self.last_yrange = (None, None)

    def create_main_panel(self):
        import platform
        if platform.system() == 'Darwin':
            from MAVProxy.modules.lib.MacOS import backend_wxagg
            FigCanvas = backend_wxagg.FigureCanvasWxAgg
        else:
            from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigCanvas
        self.panel = wx.Panel(self)

        self.init_plot()
        self.canvas = FigCanvas(self.panel, -1, self.fig)


        self.close_button = wx.Button(self.panel, -1, "Close")
        self.Bind(wx.EVT_BUTTON, self.on_close_button, self.close_button)

        self.pause_button = wx.Button(self.panel, -1, "Pause")
        self.Bind(wx.EVT_BUTTON, self.on_pause_button, self.pause_button)
        self.Bind(wx.EVT_UPDATE_UI, self.on_update_pause_button, self.pause_button)

        self.hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        self.hbox1.Add(self.close_button, border=5, flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL)
        self.hbox1.AddSpacer(1)
        self.hbox1.Add(self.pause_button, border=5, flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL)

        self.vbox = wx.BoxSizer(wx.VERTICAL)
        self.vbox.Add(self.canvas, 1, flag=wx.LEFT | wx.TOP | wx.GROW)
        self.vbox.Add(self.hbox1, 0, flag=wx.ALIGN_LEFT | wx.TOP)

        self.panel.SetSizer(self.vbox)
        self.vbox.Fit(self)

    def init_plot(self):
        self.dpi = 100
        from matplotlib.figure import Figure
        self.fig = Figure((6.0, 3.0), dpi=self.dpi)

        self.axes = self.fig.add_subplot(111)
        try:
            self.axes.set_facecolor('white')
        except AttributeError as e:
            # this was removed in matplotlib 2.2.0:
            self.axes.set_axis_bgcolor('white')

        pylab.setp(self.axes.get_xticklabels(), fontsize=8)
        pylab.setp(self.axes.get_yticklabels(), fontsize=8)

        # plot the data as a line series, and save the reference
        # to the plotted line series
        #
        self.plot_data = []
        if len(self.data[0]) == 0:
            max_y = min_y = 0
        else:
            max_y = min_y = self.data[0][0]
        for i in range(len(self.data)):
            p = self.axes.plot(
                self.data[i],
                linewidth=1,
                color=self.state.colors[i],
                label=self.state.fields[i],
                )[0]
            self.plot_data.append(p)
            if len(self.data[i]) != 0:
                min_y = min(min_y, min(self.data[i]))
                max_y = max(max_y, max(self.data[i]))

        # create X data
        self.xdata = numpy.arange(-self.state.timespan, 0, self.state.tickresolution)
        self.axes.set_xbound(lower=self.xdata[0], upper=0)
        if min_y == max_y:
            self.axes.set_ybound(min_y, max_y+0.1)
        self.axes.legend(self.state.fields, loc='upper left', bbox_to_anchor=(0, 1.1))

    def draw_plot(self):
        """ Redraws the plot
        """
        state = self.state

        if len(self.data[0]) == 0:
            print("no data to plot")
            return
        vhigh = max(self.data[0])
        vlow  = min(self.data[0])

        for i in range(1,len(self.plot_data)):
            vhigh = max(vhigh, max(self.data[i]))
            vlow  = min(vlow,  min(self.data[i]))
        ymin = vlow  - 0.05*(vhigh-vlow)
        ymax = vhigh + 0.05*(vhigh-vlow)

        if ymin == ymax:
            ymax = ymin + 0.1 * ymin
            ymin = ymin - 0.1 * ymin

        if (ymin, ymax) != self.last_yrange:
            self.last_yrange = (ymin, ymax)

            self.axes.set_ybound(lower=ymin, upper=ymax)
            #self.axes.ticklabel_format(useOffset=False, style='plain')
            self.axes.grid(True, color='gray')
            pylab.setp(self.axes.get_xticklabels(), visible=True)
            pylab.setp(self.axes.get_legend().get_texts(), fontsize='small')

        for i in range(len(self.plot_data)):
            ydata = numpy.array(self.data[i])
            xdata = self.xdata
            if len(ydata) < len(self.xdata):
                xdata = xdata[-len(ydata):]
            self.plot_data[i].set_xdata(xdata)
            self.plot_data[i].set_ydata(ydata)

        self.canvas.draw()
        self.canvas.Refresh()

    def on_pause_button(self, event):
        self.paused = not self.paused

    def on_update_pause_button(self, event):
        label = "Resume" if self.paused else "Pause"
        self.pause_button.SetLabel(label)

    def on_close_button(self, event):
        self.redraw_timer.Stop()
        self.Destroy()

    def on_idle(self, event):
        time.sleep(self.state.tickresolution*0.5)

    def on_redraw_timer(self, event):
        # if paused do not add data, but still redraw the plot
        # (to respond to scale modifications, grid change, etc.)
        #
        state = self.state
        if state.close_graph.wait(0.001):
            self.redraw_timer.Stop()
            self.Destroy()
            return
        while state.child_pipe.poll():
            state.values = state.child_pipe.recv()
        if self.paused:
            return
        for i in range(len(self.plot_data)):
            if (type(state.values[i]) == list):
                print("ERROR: Cannot plot array of length %d. Use 'graph %s[index]' instead"%(len(state.values[i]), state.fields[i]))
                self.redraw_timer.Stop()
                self.Destroy()
                return
            if state.values[i] is not None:
                self.data[i].append(state.values[i])
                while len(self.data[i]) > len(self.xdata):
                    self.data[i].pop(0)

        for i in range(len(self.plot_data)):
            if state.values[i] is None or len(self.data[i]) < 2:
                return
        self.draw_plot()



#!/usr/bin/env python
'''window layout command handling'''

from MAVProxy.modules.lib import mp_module

class LayoutModule(mp_module.MPModule):
    def __init__(self, mpstate):
        super(LayoutModule, self).__init__(mpstate, "layout", "window layout handling", public = False)
        self.add_command('layout', self.cmd_layout,
                         'window layout management',
                         ["<save|load>"])

    def cmd_layout(self, args):
        '''handle layout command'''
        from MAVProxy.modules.lib import win_layout
        if len(args) < 1:
            print("usage: layout <save|load>")
            return
        if args[0] == "load":
            win_layout.load_layout(self.mpstate.settings.vehicle_name)
        elif args[0] == "save":
            win_layout.save_layout(self.mpstate.settings.vehicle_name)

def init(mpstate):
    '''initialise module'''
    return LayoutModule(mpstate)



"""
  MAVProxy icons generator
"""
import numpy as np
from io import BytesIO
from matplotlib.figure import Figure
from matplotlib.gridspec import GridSpec
import matplotlib.path as mpath
import matplotlib.patches as mpatches
from MAVProxy.modules.lib.wx_loader import wx


class SimpleIcon():
    """
    Generate icons with Matplotlib.
    """
    def __init__(self, text=None):
        LIGHT_BLUE_BACKGROUND = "#afceff"
        VIOLET_FONT = "#4800f0"
        t1 = np.arange(0.0, 5.0, 0.1)
        t2 = np.arange(0.0, 5.0, 0.02)

        def f(t):
            return np.exp(-t) * np.cos(2*np.pi*t)

        fig = Figure(figsize=(1.28, 1.28), facecolor=LIGHT_BLUE_BACKGROUND)
        spec = GridSpec(nrows=2, ncols=1, wspace=0, hspace=0.0, height_ratios=[1, 2])
        ax = fig.add_subplot(spec[0])
        ax.text(0, 0.8, 'MAV', style='italic', fontsize=20, weight='bold', color=VIOLET_FONT)
        if text is None:
            text = "GRAPH"
        ax.text(0, 0.1, text.upper(), style='italic', fontsize=14, weight='bold', color=VIOLET_FONT)
        ax.axis('off')

        ax = fig.add_subplot(spec[1], facecolor=LIGHT_BLUE_BACKGROUND)
        if text == "GRAPH" or text == "EXPLORER":
            ax.plot(t1, f(t1), color='tab:green', marker='o')
            ax.plot(t2, f(t2), color='red')
            ax.set_xticks(t1, minor=True)
            ax.grid(True, which='both', linestyle="--")
        if text == "CONSOLE":
            ax.plot([0, 2.5], [2.5, 1.25], color='red', linewidth=4)
            ax.plot([0, 2.5], [0, 1.25], color='red', linewidth=4)
            ax.plot([3, 5], [0, 0], color='red', linewidth=4)
            ax.axis('off')
        if text == "MAP":
            path_data = [
                (mpath.Path.MOVETO, [2.5, 0]),
                (mpath.Path.LINETO, [1.25, 2.5]),
                (mpath.Path.CURVE3, [2.5, 5]),
                (mpath.Path.CURVE3, [3.75, 2.5]),
                (mpath.Path.LINETO, [3.75, 2.5]),
                (mpath.Path.CLOSEPOLY, [2.5, 0])]
            codes, verts = zip(*path_data)
            path = mpath.Path(verts, codes)

            patch = mpatches.PathPatch(path, facecolor='red', lw=2)
            ax.add_patch(patch)
            circle_neg = mpatches.Circle((2.5, 2.5), 0.5, color='white')
            ax.add_patch(circle_neg)

            ax.plot([0, 5/3.0, 10/3.0, 5], [5, 4, 5, 4], color='black', linewidth=2)
            ax.plot([0, 5/3.0, 10/3.0, 5], [-1, -2, -1, -2], color='black', linewidth=2)
            ax.plot([0, 0], [-1, 5], color='black', linewidth=2)
            ax.plot([5/3.0, 5/3.0], [-2, 4], color='black', linewidth=2)
            ax.plot([10/3.0, 10/3.0], [-1, 5], color='black', linewidth=2)
            ax.plot([5, 5], [-2, 4], color='black', linewidth=2)
            ax.set_xlim(-0.5, 5.5)
            ax.set_ylim(-2.5, 5.5)
            ax.axis('off')

        buf = BytesIO()
        fig.savefig(buf, format='png')
        buf.seek(0)
        svgimg = wx.Image(buf, wx.BITMAP_TYPE_PNG)
        self.img = svgimg

    def get_ico(self):
        """Get the ico from the image bitmap.
        need to be call after the wx.app is created."""
        return wx.Icon(wx.Bitmap(self.img))



"""
  MAVProxy terrain handling module
"""

import time

from MAVProxy.modules.mavproxy_map import mp_elevation
from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import mp_module
from MAVProxy.modules.lib import mp_settings

class TerrainModule(mp_module.MPModule):
    def __init__(self, mpstate):
        super(TerrainModule, self).__init__(mpstate, "terrain", "terrain handling", public=False)

        self.ElevationModel = mp_elevation.ElevationModel()
        self.current_request = None
        self.sent_mask = 0
        self.last_send_time = time.time()
        self.requests_received = 0
        self.blocks_sent = 0
        self.check_lat = 0
        self.check_lon = 0
        self.add_command('terrain', self.cmd_terrain, "terrain control",
                         ["<status|check>",
                          'set (TERRAINSETTING)'])
        self.terrain_settings = mp_settings.MPSettings([('debug', int, 0),
                                                        ('enable', int, 1)])
        self.add_completion_function('(TERRAINSETTING)', self.terrain_settings.completion)

    def cmd_terrain(self, args):
        '''terrain command parser'''
        usage = "usage: terrain <set|status|check>"
        if len(args) == 0:
            print(usage)
            return
        if args[0] == "status":
            print("blocks_sent: %u requests_received: %u" % (
                self.blocks_sent,
                self.requests_received))
        elif args[0] == "set":
            self.terrain_settings.command(args[1:])
        elif args[0] == "check":
            self.cmd_terrain_check(args[1:])
        else:
            print(usage)

    def cmd_terrain_check(self, args):
        '''check a piece of terrain data'''
        if len(args) >= 2:
            latlon = (float(args[0]), float(args[1]))
        else:
            latlon = self.mpstate.click_location
            if latlon is None:
                print("No map click position available")
                return
        self.check_lat = int(latlon[0]*1e7)
        self.check_lon = int(latlon[1]*1e7)
        self.master.mav.terrain_check_send(self.check_lat, self.check_lon)

    def mavlink_packet(self, msg):
        '''handle an incoming mavlink packet'''
        mtype = msg.get_type()
        master = self.master
        # add some status fields
        if mtype == 'TERRAIN_REQUEST' and self.terrain_settings.enable:
            self.current_request = msg
            self.sent_mask = 0
            self.requests_received += 1
        elif mtype == 'TERRAIN_REPORT':
            if (msg.lat == self.check_lat and
                msg.lon == self.check_lon and
                (self.check_lat != 0 or self.check_lon != 0)):
                print(msg)
                self.check_lat = 0
                self.check_lon = 0

    def send_terrain_data_bit(self, bit):
        '''send some terrain data'''
        lat = self.current_request.lat * 1.0e-7
        lon = self.current_request.lon * 1.0e-7
        bit_spacing = self.current_request.grid_spacing * 4
        (lat, lon) = mp_util.gps_offset(lat, lon,
                                        east=bit_spacing * (bit % 8),
                                        north=bit_spacing * (bit // 8))
        data = []
        for i in range(4*4):
            y = i % 4
            x = i // 4
            (lat2,lon2) = mp_util.gps_offset(lat, lon,
                                             east=self.current_request.grid_spacing * y,
                                             north=self.current_request.grid_spacing * x)
            alt = self.ElevationModel.GetElevation(lat2, lon2)
            if alt is None:
                if self.terrain_settings.debug:
                    print("no alt ", lat2, lon2)
                return
            data.append(int(alt))
        self.master.mav.terrain_data_send(self.current_request.lat,
                                          self.current_request.lon,
                                          self.current_request.grid_spacing,
                                          bit,
                                          data)
        self.blocks_sent += 1
        self.last_send_time = time.time()
        self.sent_mask |= 1<<bit
        if self.terrain_settings.debug and bit == 55:
            lat = self.current_request.lat * 1.0e-7
            lon = self.current_request.lon * 1.0e-7
            print("--lat=%f --lon=%f %.1f" % (
                lat, lon, self.ElevationModel.GetElevation(lat, lon)))
            (lat2,lon2) = mp_util.gps_offset(lat, lon,
                                             east=32*self.current_request.grid_spacing,
                                             north=28*self.current_request.grid_spacing)
            print("--lat=%f --lon=%f %.1f" % (
                lat2, lon2, self.ElevationModel.GetElevation(lat2, lon2)))


    def send_terrain_data(self):
        '''send some terrain data'''
        for bit in range(56):
            if self.current_request.mask & (1<<bit) and self.sent_mask & (1<<bit) == 0:
                self.send_terrain_data_bit(bit)
                return
        # no bits to send
        self.current_request = None
        self.sent_mask = 0

    def idle_task(self):
        '''called when idle'''
        if self.current_request is None:
            return
        if time.time() - self.last_send_time < 0.2:
            # limit to 5 per second
            return
        self.send_terrain_data()

def init(mpstate):
    '''initialise module'''
    return TerrainModule(mpstate)



'''Multiprocessing utilities

    This module contains utilties for multiprocessing such as
    custom pickle functions and class wrappers
'''

from MAVProxy.modules.lib import multiproc

import copyreg
import mmap
import os
import platform
import struct
import threading

def pickle_struct(s):
    '''Custom pickle function for struct.Struct'''

    return struct.Struct, (s.format, )

copyreg.pickle(struct.Struct, pickle_struct)

class WrapFileHandle(object):
    '''Wrap a filehandle object for pickling
    
        This is a pickleable wrapper for a filehandle object
        returned from the builtin function open(file, mode='r', ...)

        The wrapper restores the filehandle seek pointer (offset) and preserves
        the following arguments to open():

            - name (file)
            - mode
            - encoding
            - errors
            - newline
    '''
    
    def __init__(self, filehandle):
        # the filehandle is not pickled
        self._filehandle = filehandle

        # state for pickling - populated in __getstate__
        self._name = None
        self._mode = None
        self._encoding = None
        self._errors = None
        self._newlines = None
        self._offset = None

        # TODO: these are additional args to open() but it is less 
        # clear where to acquire their state when persisting
        # self._buffering = None
        # self._closefd = None
        # self._opener = None

    def unwrap(self):
        '''Returns the encapsulated object'''

        return self._filehandle

    def __getstate__(self):
        # capture the filehandle state
        self._name = self._filehandle.name
        self._mode = self._filehandle.mode
        self._encoding = self._filehandle.encoding
        self._errors = self._filehandle.errors
        self._newlines = self._filehandle.newlines
        self._offset = self._filehandle.tell()
        
        # copy the dict since we change it
        odict = self.__dict__.copy()

        # remove the _filehandle entry
        del odict['_filehandle']

        return odict

    def __setstate__(self, dict):
        # reopen file
        name = dict['_name']
        mode = dict['_mode']
        encoding = dict['_encoding']
        errors = dict['_errors']
        newlines = dict['_newlines']
        filehandle = open(name, mode=mode, encoding=encoding, errors=errors, newline=newlines)
        
        # set the seek pointer
        offset = dict['_offset']
        filehandle.seek(offset)

        # update attributes
        self.__dict__.update(dict)

        # restore the filehandle
        self._filehandle = filehandle  

class WrapMMap(object):
    '''Wrap a memory map object for pickling
    
        This is a pickleable wrapper for a mmap.mmap object
        returned from the builtin function mmap.mmap(fileno, length, ...)

        The wrapper restores the mmap's seek pointer (offset).    
    '''

    # mmap does not retain the filehandle and data_len as attributes
    # when it is constructed, so it is necessary to pass them as args
    # to the wrapper 
    def __init__(self, mm, filehandle, data_len):
        # the mmap is not pickled
        self._mm = mm

        # state for pickling - populated in __getstate__
        self._filehandle = WrapFileHandle(filehandle)
        self._data_len = data_len
        self._offset = None

    def unwrap(self):
        '''Returns the encapsulated object'''

        return self._mm

    def __getstate__(self):
        # capture the mmap state
        self._offset = self._mm.tell()
        
        # copy the dict since we change it
        odict = self.__dict__.copy()

        # remove the _mm entry
        del odict['_mm']

        return odict

    def __setstate__(self, dict):
        # reopen mmap
        filehandle = dict['_filehandle'].unwrap()
        data_len = dict['_data_len']

        mm = None
        if platform.system() == "Windows":
            mm = mmap.mmap(filehandle.fileno(), data_len, None, mmap.ACCESS_READ)
        else:
            mm = mmap.mmap(filehandle.fileno(), data_len, mmap.MAP_PRIVATE, mmap.PROT_READ)

        # set the seek pointer
        offset = dict['_offset']
        mm.seek(offset)

        # update attributes
        self.__dict__.update(dict)

        # restore the mmap
        self._mm = mm  

mutex = multiproc.Lock()

class MPChildTask(object):
    '''Manage a MAVProxy child task
    
        MAVProxy child tasks typically require access to the dataflash
        or telemetry logs. For processes started using the `spawn`
        start method this requires the arguments to the child process
        to be pickleable, which is not the case for file handles and 
        memory mapped files. 

        This class provides two functions `wrap` and `unwrap` that are
        called before and after the parent spawns the child process,
        and in the child process itself before the public child task
        is called. Custom pickle functions and class wrappers / unwrappers
        should be placed in these two functions.
    '''

    def __init__(self, *args, **kwargs):
        self._child_pipe_recv, self._parent_pipe_send = multiproc.Pipe(duplex=False)
        self._close_event = multiproc.Event()
        self._close_event.clear()
        self._child = None

    # @abstractmethod
    def wrap(self):
        '''Apply custom pickle wrappers to non-pickleable attributes'''
        pass

    # @abstractmethod
    def unwrap(self):
        '''Unwrap custom pickle wrappers of non-pickleable attributes'''
        pass

    # @abstractmethod
    def child_task(self):
        '''Launch the child function or application'''
        pass

    def start(self):
        '''Apply custom pickle wrappers then start the child process'''

        # apply lock while the task is started as wrapping may mutate state
        with mutex:
            self.wrap()
            try:
                if os.name == 'nt':
                    # Windows can't pickle the mavlog object, thus can't spin off
                    self._child = threading.Thread(target=self._child_task)
                else:
                    self._child = multiproc.Process(target=self._child_task)
                self._child.start()
            finally:
                if not os.name == 'nt':
                    self.unwrap()

        # deny the parent access to the child end of the pipe
        self.child_pipe_recv.close()

    def _child_task(self):
        '''A non-public child task that calls unwrap before running the public child task'''
        
        # deny the child access to the parent end of the pipe and unwrap
        self.parent_pipe_send.close()
        self.unwrap()

        # call child task (in sub-class)
        self.child_task()

        # set the close event when the task is complete
        self.close_event.set()

    @property
    def child_pipe_recv(self):
        '''The child end of the pipe for receiving data from the parent'''

        return self._child_pipe_recv

    @property
    def parent_pipe_send(self):
        '''The parent end of the pipe for sending data to the child'''

        return self._parent_pipe_send

    @property
    def close_event(self):
        '''A multiprocessing close event'''

        return self._close_event

    @property
    def child(self):
        '''The child process'''

        return self._child

    def close(self):
        '''Set the close event and join the process'''

        self.close_event.set()
        if self.is_alive():
            self.child.join(2)

    def is_alive(self):
        '''Returns True if the child process is alive'''

        return self.child.is_alive()

class MPDataLogChildTask(MPChildTask):
    '''Manage a MAVProxy child task that expects a dataflash or telemetry log'''

    def __init__(self, *args, **kwargs):
        '''
        Parameters
        ----------
        mlog : DFReader
            A dataflash or telemetry log
        '''
        super(MPDataLogChildTask, self).__init__(*args, **kwargs)

        # all attributes are implicitly passed to the child process 
        self._mlog = kwargs['mlog']

    # @override
    def wrap(self):
        '''Apply custom pickle wrappers to non-pickleable attributes'''

        # wrap filehandle and mmap in mlog for pickling
        filehandle = self._mlog.filehandle
        data_map = self._mlog.data_map
        data_len = self._mlog.data_len
        self._mlog.filehandle = WrapFileHandle(filehandle)
        self._mlog.data_map = WrapMMap(data_map, filehandle, data_len)

    # @override
    def unwrap(self):
        '''Unwrap custom pickle wrappers of non-pickleable attributes'''

        # restore the state of mlog
        self._mlog.filehandle = self._mlog.filehandle.unwrap()
        self._mlog.data_map = self._mlog.data_map.unwrap()

    @property
    def mlog(self):
        '''The dataflash log (DFReader)'''

        return self._mlog



#!/usr/bin/env python
'''
OSD Module
Andy Piper, August 2020, Vendee, France

'''

import os
import os.path
import sys
from pymavlink import mavutil
import errno
import time

from MAVProxy.modules.lib import mp_module
from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import mp_settings

class osd(mp_module.MPModule):
    def __init__(self, mpstate):
        """Initialise OSD module"""
        super(osd, self).__init__(mpstate, "osd", "")

        self.request_id = 1
        self.add_command('osd', self.cmd_osd, "OSD module",
            ['param-set <5|6> <1|2|3|4|5|6|7|8|9> (PARAMETER) (TYPES)',
             'param-show <5|6> <1|2|3|4|5|6|7|8|9>'
            ])
        self.add_completion_function('(TYPES)', self.param_type_completion)
        self.type_map = {
            mavutil.mavlink.OSD_PARAM_NONE : "NONE",
            mavutil.mavlink.OSD_PARAM_SERIAL_PROTOCOL : "SERIAL_PROTOCOL",
            mavutil.mavlink.OSD_PARAM_SERVO_FUNCTION : "SERVO_FUNCTION",
            mavutil.mavlink.OSD_PARAM_AUX_FUNCTION : "AUX_FUNCTION",
            mavutil.mavlink.OSD_PARAM_FLIGHT_MODE : "FLIGHT_MODE",
            mavutil.mavlink.OSD_PARAM_FAILSAFE_ACTION : "FAILSAFE_ACTION",
            mavutil.mavlink.OSD_PARAM_FAILSAFE_ACTION_1 : "FAILSAFE_ACTION_1",
            mavutil.mavlink.OSD_PARAM_FAILSAFE_ACTION_2 : "FAILSAFE_ACTION_2" }
        self.invtype_map = { v : k for k, v in self.type_map.items()}
        self.param_list = {}

    def usage(self):
        '''show help on command line options'''
        return "Usage: osd <param-set|param-show|set>"

    def cmd_osd(self, args):
        '''control behaviour of the OSD module'''
        if len(args) == 0:
            print(self.usage())
        elif args[0] == "param-set":
            self.param_set(args[1:])
        elif args[0] == "param-show":
            self.param_show(args[1:])
        else:
            print(self.usage())

    def param_set(self, args):
        '''sets an OSD parameter'''
        if len(args) < 3 or len(args) > 6:
            print("param-set <screen> <index> <name> (<type> | <min> <max> <increment>)")
            return
        screen = int(args[0], 0)
        index = int(args[1], 0)
        name = args[2]
        type = mavutil.mavlink.OSD_PARAM_NONE
        min_value = 0.
        max_value = 0.
        increment = 0.
        # config type implies the ranges are pre-defined
        if len(args) > 3:
            type = self.string_to_config_type(args[3])
            # can't have config type and ranges
            if type is not None and len(args) > 4:
                print("param-set <screen> <index> <name> (<type> | <min> <max> <increment>)")
                return

        if len(args) > 3 and type is None:
            type = mavutil.mavlink.OSD_PARAM_NONE
            min_value = float(args[3])
            if len(args) > 4:
                max_value = float(args[4])
            if len(args) > 5:
                increment = float(args[5])

        if sys.version_info.major >= 3:
            name = bytearray(name, 'ascii')

        self.master.mav.osd_param_config_send(self.target_system,
                                        self.target_component,
                                        self.request_id,
                                        screen,
                                        index,
                                        name,
                                        type,
                                        min_value,
                                        max_value,
                                        increment)
        self.request_id += 1

    def param_show(self, args):
        '''show an OSD parameter or list of parameters'''
        if len(args) == 0:
            for screen in range(5, 7):
                for index in range(1, 10):
                    self.master.mav.osd_param_show_config_send(self.target_system,
                                                    self.target_component,
                                                    self.request_id,
                                                    screen,
                                                    index)
                    self.request_id += 1
            return

        elif len(args) == 1:
            screen = int(args[0], 0)
            for index in range(1, 10):
                self.master.mav.osd_param_show_config_send(self.target_system,
                                                self.target_component,
                                                self.request_id,
                                                screen,
                                                index)
                self.request_id += 1
            return

        elif len(args) != 2:
            print("param-show <screen> <index>")
            return
        screen = int(args[0], 0)
        index = int(args[1], 0)

        self.master.mav.osd_param_show_config_send(self.target_system,
                                        self.target_component,
                                        self.request_id,
                                        screen,
                                        index)
        self.request_id += 1

    def mavlink_packet(self, m):
        '''handle mavlink packets'''
        mtype = m.get_type()
        if mtype == "OSD_PARAM_CONFIG_REPLY" or mtype == "OSD_PARAM_SHOW_CONFIG_REPLY":
            if m.result == mavutil.mavlink.OSD_PARAM_INVALID_PARAMETER:
                print("OSD request %u failed: invalid parameter" % (m.request_id))
            elif m.result != 0:
                print("OSD request %u failed: %u" % (m.request_id, m.result))
            else:
                if mtype == "OSD_PARAM_CONFIG_REPLY":
                    print("OSD parameter set")
                else:
                    print("%s %f %f %f %s" % (m.param_id, m.min_value, m.max_value, m.increment,
                        self.config_type_to_string(m.config_type)))

    def config_type_to_string(self, config_type):
        '''convert config type to a string'''
        if config_type in self.type_map:
            return self.type_map[config_type]
        return None

    def string_to_config_type(self, config_str):
        '''convert a string to a config type'''
        if config_str in self.invtype_map:
            return self.invtype_map[config_str]
        return None

    def param_type_completion(self, text):
        '''return list of param-set type completions'''
        return self.invtype_map.keys()

def init(mpstate):
    '''initialise OSD module'''
    return osd(mpstate)



'''
variable substitution in strings using a dictionary
'''

import sys

class MAVSubstituteError(Exception):
    def __init__(self, message, inner_exception=None):
        self.message = message
        self.inner_exception = inner_exception
        self.exception_info = sys.exc_info()
    def __str__(self):
        return self.message

class MAVSubstitute(object):
    '''simple templating system'''
    def __init__(self,
                 start_var_token="${", 
                 end_var_token="}", 
                 checkmissing=True):
        self.start_var_token = start_var_token
        self.end_var_token = end_var_token
        self.checkmissing = checkmissing

    def find_end(self, text, start_token, end_token, ignore_end_token=None):
        '''find the of a token.
        Returns the offset in the string immediately after the matching end_token'''
        if not text.startswith(start_token):
            raise MAVSubstituteError("invalid token start")
        offset = len(start_token)
        nesting = 1
        while nesting > 0:
            idx1 = text[offset:].find(start_token)
            idx2 = text[offset:].find(end_token)
            # Check for false positives due to another similar token
            # For example, make sure idx2 points to the second '}' in ${{field: ${name}}}
            if ignore_end_token:
                combined_token = ignore_end_token + end_token
                if text[offset+idx2:offset+idx2+len(combined_token)] == combined_token:
                    idx2 += len(ignore_end_token)
            if idx1 == -1 and idx2 == -1:
                raise MAVSubstituteError("token nesting error")
            if idx1 == -1 or idx1 > idx2:
                offset += idx2 + len(end_token)
                nesting -= 1
            else:
                offset += idx1 + len(start_token)
                nesting += 1
        return offset

    def find_var_end(self, text):
        '''find the of a variable'''
        return self.find_end(text, self.start_var_token, self.end_var_token)

    def substitute(self, text, subvars={},
                   checkmissing=None):
        '''substitute variables in a string'''

        if checkmissing is None:
            checkmissing = self.checkmissing

        while True:
            idx = text.find(self.start_var_token)
            if idx == -1:
                return text
            endidx = text[idx:].find(self.end_var_token)
            if endidx == -1:
                raise MAVSubstituteError('missing end of variable: %s' % text[idx:idx+10])
            varname = text[idx+2:idx+endidx]
            fullvar = varname

            # allow default value after a :
            def_value = None
            colon = varname.find(':')
            if colon != -1:
                def_value = varname[colon+1:]
                varname = varname[:colon]

            if varname in subvars:
                value = subvars[varname]
            elif def_value is not None:
                value = def_value
            elif checkmissing:
                raise MAVSubstituteError("unknown variable in '%s%s%s'" % (
                    self.start_var_token, varname, self.end_var_token))
            else:
                return text[0:idx+endidx] + self.substitute(text[idx+endidx:], subvars, checkmissing=False)
            text = text.replace("%s%s%s" % (self.start_var_token, fullvar, self.end_var_token), str(value))
        return text

if __name__ == "__main__":
    import os
    sub = MAVSubstitute()
    for v in sys.argv[1:]:
        print("'%s' -> '%s'" % (v, sub.substitute(v, os.environ)))



import time
import math
import cv2
from pymavlink import mavutil
from droneapi.lib import VehicleMode, Location
import sc_config
from sc_video import sc_video
from sc_webcam import SmartCameraWebCam
from sc_SonyQX1 import SmartCamera_SonyQX

"""
sc_main.py - runs top level smart camera function

To run this module:
* Run mavproxy.py with the correct options to connect to your vehicle
* module load api
* api start sc_main.py

(Once tested we can put these directives into a mavinit.scr file and mavproxy will load/run
    this code automatically)

"""

class SmartCamera(object):
    def __init__(self, use_api):

        # if using droneapi connect to vehicle
        if (use_api):
            # First get an instance of the API endpoint (the connect via web case will be similar)
            self.api = local_connect()

            # Our vehicle (we assume the user is trying to control the virst vehicle attached to the GCS)
            self.vehicle = self.api.get_vehicles()[0]
        else:
            self.api = None
            self.vehicle = None

        # initialised flag
        self.home_initialised = False
        # timer to intermittently check for home position
        self.last_home_check = time.time()
        self.home_location = None

        # vehicle mission
        self.mission_cmds = None

        # check if we should display debug messages
        self.debug = sc_config.config.get_boolean('general','debug',True)

        # register cameras
        self.register_cameras();

        # initialise video writer
        self.writer = None

    # register cameras - creates camera objects based on camera-type configuration
    def register_cameras(self):

        # initialise list
        self.camera_list = []

        #look for up to 2 cameras
        for i in range(0,2):
            config_group = "camera%d" % i
            camera_type = sc_config.config.get_integer(config_group, 'type', 0)
            # webcam
            if camera_type == 1:
                new_camera = SmartCameraWebCam(i)
                self.camera_list = self.camera_list + [new_camera]

            # Sony QX1
            if camera_type == 2:
                new_camera = SmartCamera_SonyQX(i,"wlan0")
                if new_camera.boValidCameraFound() is True:
                    self.camera_list = self.camera_list + [new_camera]
                    print("Found QX Camera")

        # display number of cameras found
        print ("cameras found: %d" % len(self.camera_list))

    # fetch_mission - fetch mission from flight controller
    def fetch_mission(self):
        # download the vehicle waypoints
        self.mission_cmds = self.vehicle.commands
        self.mission_cmds.download()
        self.mission_cmds.wait_valid()

    # check home - intermittently checks for changes to the home location
    def check_home(self):

        # return immediately if home has already been initialised
        if self.home_initialised:
            return True

        # check for home no more than once every two seconds
        if (time.time() - self.last_home_check > 2):

            # update that we have performed a status check
            self.last_home_check = time.time()

            # check if we have a vehicle
            if self.vehicle is None:
                self.vehicle = self.api.get_vehicles()[0]
                return

            # ensure the vehicle's position is known
            if self.vehicle.location is None:
                return False
            if self.vehicle.location.lat is None or self.vehicle.location.lon is None or self.vehicle.location.alt is None:
                return False

            # download the vehicle waypoints if we don't have them already
            if self.mission_cmds is None:
                self.fetch_mission()
                return False

            # get the home lat and lon
            home_lat = self.mission_cmds[0].x
            home_lon = self.mission_cmds[0].y
            home_alt = self.mission_cmds[0].z

            # sanity check the home position
            if home_lat is None or home_lon is None or home_alt is None:
                return False

            # sanity check again and set home position
            if (home_lat != 0 and home_lon != 0):
                self.home_location = Location(home_lat,home_lon,home_alt)
                self.home_initialised = True
            else:
                self.mission_cmds = None

            # To-Do: if we wish to have the same home position as the flight controller
            # we must download the home waypoint again whenever the vehicle is armed

        # return whether home has been initialised or not
        return self.home_initialised

    # checks if video output should be started
    def check_video_out(self):

        # return immediately if video has already been started
        if not self.writer is None:
            return

        # start video once vehicle is armed
        if self.vehicle.armed:
            self.writer = sc_video.open_video_writer()

    # check_status - poles vehicle' status to determine if we should record video or not
    def check_status(self):

        # download the vehicle waypoints if we don't have them already
        # To-Do: do not load waypoints if vehicle is armed
        if self.mission_cmds is None:
            self.fetch_mission()
            return

    # take_picture_all - ask all cameras to take a picture
    def take_picture_all(self):
        for cam in self.camera_list:
            cam.take_picture()

    # saves_picture_all - ask all cameras for their latest image and saves to files
    def save_picture_all(self):
        cam_num = 0
        for cam in self.camera_list:
            img = cam.get_latest_image()
            # display image
            window_name = "cam%d" % cam_num
            cv2.namedWindow(window_name, 0)
            cv2.resizeWindow(window_name, 640, 480)
            cv2.imshow (window_name, img)
            # write to file
            #imgfilename = "C:\Users\rmackay9\Documents\GitHub\ardupilot-balloon-finder\smart_camera\img%d-%d.jpg" % (cam_num,cam.get_image_counter())
            #imgfilename = "img%d-%d.jpg" % (cam_num,cam.get_image_counter())
            #print (imgfilename)
            #cv2.imwrite(imgfilename, img)

            # check for ESC key being pressed
            k = cv2.waitKey(5) & 0xFF
            if k == 27:
                break
            cam_num = cam_num + 1

    # get image from sc_video class and write to file
    def analyze_image(self):

        # record time
        now = time.time()

        # get new image from camera
        f = self.get_frame()

        # save image for debugging later
        if not self.writer is None:
            self.writer.write(f)

    def run(self):
        while True:
            # ask all cameras to take a picture
            self.take_picture_all()

            # store images to disk
            self.save_picture_all()

            # Don't suck up too much CPU, only process a new image occasionally
            time.sleep(1)

        '''
        while not self.api.exit:

            # only process images once home has been initialised
            if self.check_home():

                # start video if required
                self.check_video_out()

                # check if we are controlling the vehicle
                self.check_status()

                # ask all cameras to take a picture
                self.take_picture_all()

            # Don't suck up too much CPU, only process a new image occasionally
            time.sleep(2.0)

        if not self.use_simulator:
            sc_video.stop_background_capture()
        '''

# initialise depending upon whether running from command line or as part of mavproxy
if __name__ == "__main__":
    sc_main = SmartCamera(False)
else:
    sc_main = SmartCamera(True)

# run the smart camera
sc_main.run()




'''
Graphical editing of graph definition
'''
from MAVProxy.modules.lib.wx_loader import wx
from MAVProxy.modules.lib.graphdefinition import GraphDefinition

class GraphDialog(wx.Dialog):
    def __init__(self, title, graphdef, callback):
        wx.Dialog.__init__(self, None, -1, title, size=(900, 400))

        self.callback = callback
        self.graphdef = graphdef

        self.panel = wx.Panel(self, -1)
        vbox = wx.BoxSizer(wx.VERTICAL)

        # name entry
        hbox_name = wx.BoxSizer(wx.HORIZONTAL)
        st_name   = wx.StaticText(self.panel, -1, 'Name: ')
        self.tc_name = wx.TextCtrl(self.panel, -1, size=(400, -1))
        try:
            self.tc_name.Value = self.graphdef.name
        except Exception:
            self.tc_name.Value = 'UNKNOWN'
        hbox_name.Add(st_name, 0, wx.LEFT, 10)
        hbox_name.Add(self.tc_name, 0, wx.LEFT, 35)
        vbox.Add(hbox_name, 0, wx.TOP, 10)

        # expression entry
        st = wx.StaticText(self.panel, -1, 'Expressions: ')
        vbox.Add(st, 0, wx.LEFT, 10)

        hbox_expressions = wx.BoxSizer(wx.HORIZONTAL)
        self.tc_expressions = wx.TextCtrl(self.panel, -1, style=wx.TE_MULTILINE|wx.HSCROLL, size=(800, 80))
        elist = []
        for e in self.graphdef.expressions:
            e = ' '.join(e.split())
            elist.append(e)
        self.tc_expressions.Value = '\n'.join(elist)
        vbox.Add(self.tc_expressions, 0, wx.LEFT, 15)

        # description entry
        st = wx.StaticText(self.panel, -1, 'Description: ')
        vbox.Add(st, 0, wx.LEFT, 10)
        self.tc_description = wx.TextCtrl(self.panel, -1, style=wx.TE_MULTILINE)
        vbox.Add(self.tc_description, 1, wx.EXPAND | wx.TOP | wx.RIGHT | wx.LEFT, 15)
        self.tc_description.Value = self.graphdef.description

        # buttons
        button_save   = wx.Button(self.panel, 1, 'Save')
        button_cancel = wx.Button(self.panel, 2, 'Cancel')
        button_test = wx.Button(self.panel, 3, 'Test')
        hbox_buttons = wx.BoxSizer(wx.HORIZONTAL)
        hbox_buttons.Add(button_save,   0, wx.LEFT, 10)
        hbox_buttons.Add(button_cancel, 0, wx.LEFT, 10)
        hbox_buttons.Add(button_test, 0, wx.LEFT, 10)
        vbox.Add(hbox_buttons, 0, wx.TOP, 10)
        self.Bind(wx.EVT_BUTTON, self.OnSave, id=1)
        self.Bind(wx.EVT_BUTTON, self.OnCancel, id=2)
        self.Bind(wx.EVT_BUTTON, self.OnTest, id=3)

        self.panel.SetSizer(vbox)
        self.Centre()

    def update_values(self):
        self.graphdef.name = self.tc_name.Value.strip()
        self.graphdef.expressions = self.tc_expressions.Value.split('\n')
        self.graphdef.description = self.tc_description.Value

    def OnCancel(self, event):
        self.Close()

    def OnTest(self, event):
        self.update_values()
        self.callback('test', self.graphdef)

    def OnSave(self, event):
        self.update_values()
        self.callback('save', self.graphdef)



class Attitude():
    '''The current Attitude Data'''
    def __init__(self, attitudeMsg):
        self.pitch = attitudeMsg.pitch
        self.roll = attitudeMsg.roll
        self.yaw = attitudeMsg.yaw

class VFR_HUD():
    '''HUD Information.'''
    def __init__(self,hudMsg):
        self.airspeed = hudMsg.airspeed
        self.groundspeed = hudMsg.groundspeed
        self.heading = hudMsg.heading
        self.throttle = hudMsg.throttle
        self.climbRate = hudMsg.climb
        
class Global_Position_INT():
    '''Altitude relative to ground (GPS).'''
    def __init__(self,gpsINT,curTime):
        self.relAlt = gpsINT.relative_alt/1000.0
        self.curTime = curTime
        
class BatteryInfo():
    '''Voltage, current and remaning battery.'''
    def __init__(self,batMsg):
        self.voltage = batMsg.voltage_battery/1000.0 # Volts
        self.current = batMsg.current_battery/100.0 # Amps
        self.batRemain = batMsg.battery_remaining # %
        
class FlightState():
    '''Mode and arm state.'''
    def __init__(self,mode,armState):
        self.mode = mode
        self.armState = armState
        
class WaypointInfo():
    '''Current and final waypoint numbers, and the distance
    to the current waypoint.'''
    def __init__(self,current,final,currentDist,nextWPTime,wpBearing):
        self.current = current
        self.final = final
        self.currentDist = currentDist
        self.nextWPTime = nextWPTime
        self.wpBearing = wpBearing
        
class FPS():
    '''Stores intended frame rate information.'''
    def __init__(self,fps):
        self.fps = fps # if fps is zero, then the frame rate is unrestricted
        
        
        


"""
  MAVProxy sailing dashboard gui elements
"""

from MAVProxy.modules.lib.wx_loader import wx
from MAVProxy.modules.lib.wxsaildash_util import WindReference, SpeedUnit, WindAngleAndSpeed, WaterSpeedAndHeading

import wx.lib.agw.speedmeter as SM

import math
import time

class SailingDashboardFrame(wx.Frame):
    '''The main frame of the sailing dashboard'''

    def __init__(self, state, title, size):
        super(SailingDashboardFrame, self).__init__(None, title=title, size=size)
        self._state = state
        self._title = title

        # control update rate
        self._timer = wx.Timer(self)
        self._fps = 10.0
        self._start_time = time.time()

        # events
        self.Bind(wx.EVT_TIMER, self.OnTimer, self._timer)
        self.Bind(wx.EVT_IDLE, self.OnIdle)
        self.Bind(wx.EVT_CHAR_HOOK, self.OnKeyPress)

        # restart the timer
        self._timer.Start(milliseconds=100)

        # create wind meters for true and apparent wind
        self._wind_meter1 = WindMeter(self)
        self._wind_meter1.SetWindReference(WindReference.RELATIVE)
        self._wind_meter1.SetWindSpeedUnit(SpeedUnit.KNOTS)
        self._wind_meter1.SetWindAngle(0)
        self._wind_meter1.SetWindSpeed(0)

        self._wind_meter2 = WindMeter(self)
        self._wind_meter2.SetWindReference(WindReference.TRUE)
        self._wind_meter2.SetWindSpeedUnit(SpeedUnit.KNOTS)
        self._wind_meter2.SetWindAngle(0)
        self._wind_meter2.SetWindSpeed(0)

        # instrument display
        self._instr1 = InstrumentDisplay(self)
        self._instr1.label = "AWS"
        self._instr1.unit = "kn"
        self._instr1.value = 0.0

        self._instr2 = InstrumentDisplay(self)
        self._instr2.label = "TWS"
        self._instr2.unit = "kn"
        self._instr2.value = 0.0

        self._instr3 = InstrumentDisplay(self)
        self._instr3.label = "STW"
        self._instr3.unit = "kn"
        self._instr3.value = 0.0

        self._instr4 = InstrumentDisplay(self)
        self._instr4.label = "AWA"
        self._instr4.unit = "deg"
        self._instr4.value = 0.0

        self._instr5 = InstrumentDisplay(self)
        self._instr5.label = "TWA"
        self._instr5.unit = "deg"
        self._instr5.value = 0.0

        self._instr6 = InstrumentDisplay(self)
        self._instr6.label = "HDT"
        self._instr6.unit = "deg"
        self._instr3.value = 0.0

        # instrument panel sizers
        self._instr_sizer1 = wx.BoxSizer(wx.VERTICAL) 
        self._instr_sizer1.Add(self._instr1, 1, wx.EXPAND)
        self._instr_sizer1.Add(self._instr2, 1, wx.EXPAND)
        self._instr_sizer1.Add(self._instr3, 1, wx.EXPAND)

        self._instr_sizer2 = wx.BoxSizer(wx.VERTICAL) 
        self._instr_sizer2.Add(self._instr4, 1, wx.EXPAND)
        self._instr_sizer2.Add(self._instr5, 1, wx.EXPAND)
        self._instr_sizer2.Add(self._instr6, 1, wx.EXPAND)

        # top level sizers
        self._top_sizer = wx.BoxSizer(wx.HORIZONTAL) 
        self._top_sizer.Add(self._wind_meter1, 2, wx.EXPAND)
        self._top_sizer.Add(self._wind_meter2, 2, wx.EXPAND)        
        self._top_sizer.Add(self._instr_sizer1, 1, wx.EXPAND)
        self._top_sizer.Add(self._instr_sizer2, 1, wx.EXPAND)
        
        # layout sizers
        self.SetSizer(self._top_sizer)
        self.SetAutoLayout(1)
        # self.sizer.Fit(self)

    def OnIdle(self, event):
        '''Handle idle events
        
            - e.g. managed scaling when window resized if needed
        ''' 
        
        pass

    def OnTimer(self, event):
        '''Handle timed tasks'''
        
        # check for close events
        if self._state.close_event.wait(timeout=0.001):
            # stop the timer and destroy the window - this ensures
            # the window is closed whhen the module is unloaded
            self._timer.Stop()
            self.Destroy()
            return

        # receive data from the module
        while self._state.child_pipe_recv.poll():
            obj_list = self._state.child_pipe_recv.recv()
            for obj in obj_list:
                if isinstance(obj, WindAngleAndSpeed):
                    if obj.wind_reference == WindReference.RELATIVE:
                        # apparent wind meter
                        self._wind_meter1.SetWindReference(obj.wind_reference)
                        self._wind_meter1.SetWindAngle(obj.wind_angle)
                        self._wind_meter1.SetWindSpeed(obj.wind_speed)
                        self._wind_meter1.SetWindSpeedUnit(obj.wind_speed_unit)

                        # apparent wind displays    
                        self._instr1.value = obj.wind_speed
                        self._instr4.value = obj.wind_angle

                    elif obj.wind_reference == WindReference.TRUE:
                        # apparent wind meter
                        self._wind_meter2.SetWindReference(obj.wind_reference)
                        self._wind_meter2.SetWindAngle(obj.wind_angle)
                        self._wind_meter2.SetWindSpeed(obj.wind_speed)
                        self._wind_meter2.SetWindSpeedUnit(obj.wind_speed_unit)

                        # apparent wind displays    
                        self._instr2.value = obj.wind_speed
                        self._instr5.value = obj.wind_angle

                elif isinstance(obj, WaterSpeedAndHeading):
                    # water speed and heading     
                    self._instr3.value = obj.water_speed
                    self._instr6.value = obj.heading_true


        # reset counters
        self._start_time = time.time()

    def OnKeyPress(self, event):
        '''Handle keypress events'''

        pass

class WindMeter(SM.SpeedMeter):
    ''' 
    class: `WindMeter` is a custom `SpeedMeter`for displaying true or apparent wind 
    '''

    def __init__(self, *args, **kwargs):
        # customise the agw style
        kwargs.setdefault('agwStyle', SM.SM_DRAW_HAND
            |SM.SM_DRAW_PARTIAL_SECTORS
            |SM.SM_DRAW_SECONDARY_TICKS
            |SM.SM_DRAW_MIDDLE_TEXT)

        # initialise super class
        super(WindMeter, self).__init__(*args, **kwargs)

        # non-public attributes 
        self._wind_reference = WindReference.RELATIVE
        self._wind_speed = 0
        self._wind_speed_unit = SpeedUnit.KNOTS

        # colours
        dial_colour = wx.ColourDatabase().Find('DARK SLATE GREY')
        port_arc_colour = wx.ColourDatabase().Find('RED')
        stbd_arc_colour = wx.ColourDatabase().Find('GREEN')
        bottom_arc_colour = wx.ColourDatabase().Find('TAN')
        self.SetSpeedBackground(dial_colour)

        # set lower limit to 2.999/2 to prevent the partial sectors from filling
        # the entire dial
        self.SetAngleRange(- 2.995/2 * math.pi, math.pi/2)

        # create the intervals
        intervals = range(0, 361, 20)
        self.SetIntervals(intervals)

        # assign colours to sectors
        colours = [dial_colour] \
            + [stbd_arc_colour] * 2 \
            + [dial_colour] * 4 \
            + [bottom_arc_colour] * 4 \
            + [dial_colour] * 4 \
            + [port_arc_colour] * 2 \
            + [dial_colour]
        self.SetIntervalColours(colours)

        # assign the ticks, colours and font
        ticks = ['0', '20', '40', '60', '80', '100', '120', '140', '160', '180', '-160', '-140', '-120', '-100', '-80', '-60', '-40', '-20', '']
        self.SetTicks(ticks)
        self.SetTicksColour(wx.WHITE)
        self.SetNumberOfSecondaryTicks(1)
        self.SetTicksFont(wx.Font(12, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))

        # set the colour for the first hand indicator
        self.SetHandColour(wx.Colour(210, 210, 210))
        self.SetHandStyle("Hand")

        # do not draw the external (container) arc
        self.DrawExternalArc(False)

        # initialise the meter to zero
        self.SetWindAngle(0.0)

        # wind speed display
        self.SetMiddleTextColour(wx.WHITE)
        self.SetMiddleTextFont(wx.Font(16, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        self.SetWindSpeed(0.0)

    def SetWindAngle(self, wind_angle):
        '''Set the wind angle

            The wind angle is measured with respect to the front of the vehicle, so: 
                -180 <= wind_angle <= 180
        '''
        
        # Convert the wind angle to the meter interval [0 - 360] 
        dial_value = wind_angle 
        if wind_angle < 0:
            dial_value += 360

        self.SetSpeedValue(dial_value)

    def SetWindSpeed(self, wind_speed):
        '''Set the wind speed'''

        self._wind_speed = wind_speed
        
        reference_code = None
        if self._wind_reference == WindReference.RELATIVE:
            reference_code ='AWS'
        elif self._wind_reference == WindReference.TRUE:
            reference_code ='TWS'
        else:
            reference_code ='INVALID'

        unit_code = None
        if self._wind_speed_unit == SpeedUnit.KPH:
            unit_code ='kph'
        elif self._wind_speed_unit == SpeedUnit.MPH:
            unit_code ='mph'
        elif self._wind_speed_unit == SpeedUnit.KNOTS:
            unit_code ='kn'
        else:
            unit_code ='INVALID'

        txt = reference_code + "  {:.2f}".format(self._wind_speed) + ' ' + unit_code
        self.SetMiddleText(txt)

    def SetWindReference(self, reference):
        '''Set the wind reference (i.e relative(apparent) or true)'''

        if not (reference == WindReference.RELATIVE or reference == WindReference.TRUE):
            raise Exception("Invalid wind reference. Must be 'RELATIVE' or 'TRUE'")
            
        self._wind_reference = reference

    def SetWindSpeedUnit(self, unit):
        '''Set the wind speed units'''
        
        if not (unit == SpeedUnit.KPH or unit == SpeedUnit.MPH or unit == SpeedUnit.KNOTS):
            raise Exception("Invalid wind speed unit. Must be 'KPH' or 'MPH' or 'KNOTS'")
            
        self._wind_speed_unit = unit

class InstrumentDisplay(wx.Panel):
    '''
    class: `InstrumentDisplay` is a panel for displaying sailing instrument data
    '''

    def __init__(self, *args, **kwargs):
        # customise the style
        kwargs.setdefault('style', wx.TE_READONLY)

        # initialise super class
        super(InstrumentDisplay, self).__init__(*args, **kwargs)

        # set the background colour (also for text controls)
        # self.SetBackgroundColour(wx.WHITE)

        # text controls
        self._label_text = wx.TextCtrl(self, style=wx.BORDER_NONE)
        self._label_text.SetFont(wx.Font(12, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        self._label_text.SetBackgroundColour(self.GetBackgroundColour())
        self._label_text.ChangeValue("---")

        self._value_text = wx.TextCtrl(self, style=wx.BORDER_NONE|wx.TE_CENTRE)
        self._value_text.SetFont(wx.Font(30, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        self._value_text.SetBackgroundColour(self.GetBackgroundColour())
        self._value_text.ChangeValue("-.-")
        self._value_text.SetMinSize((60, 40))

        self._unit_text = wx.TextCtrl(self, style=wx.BORDER_NONE|wx.TE_RIGHT)
        self._unit_text.SetFont(wx.Font(12, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        self._unit_text.SetBackgroundColour(self.GetBackgroundColour())
        self._unit_text.ChangeValue("--")

        # value text needs a nested sizer to centre vertically
        self._value_sizer = wx.BoxSizer(wx.HORIZONTAL)
        self._value_sizer.Add(self._value_text,
            wx.SizerFlags(1).Align(wx.ALIGN_CENTRE_VERTICAL).Border(wx.ALL, 0))

        # top level sizers
        self._top_sizer = wx.BoxSizer(wx.VERTICAL) 
        self._top_sizer.Add(self._label_text,
            wx.SizerFlags(0).Align(wx.ALIGN_TOP|wx.ALIGN_LEFT).Border(wx.ALL, 2))
        self._top_sizer.Add(self._value_sizer,
            wx.SizerFlags(1).Expand().Border(wx.ALL, 1))
        self._top_sizer.Add(self._unit_text,
            wx.SizerFlags(0).Align(wx.ALIGN_RIGHT).Border(wx.ALL, 2))

        # layout sizers
        self._top_sizer.SetSizeHints(self)
        self.SetSizer(self._top_sizer)
        self.SetAutoLayout(True)

    @property
    def label(self):
        return self._label_text.GetValue()

    @label.setter
    def label(self, label):
        self._label_text.ChangeValue(label)

    @property
    def value(self):
        return self._value_text.GetValue()

    @value.setter
    def value(self, value):
        self._value_text.ChangeValue("{:.2f}".format(value))

    @property
    def unit(self):
        return self._unit_text.GetValue()

    @unit.setter
    def unit(self, unit):
        self._unit_text.ChangeValue(unit)



import os
import sys
import webbrowser

import mmap_server

g_module_context = None

from MAVProxy.modules.lib import mp_module

class MMapModule(mp_module.MPModule):
    def __init__(self, mpstate):
        super(MMapModule, self).__init__(mpstate, 'mmap', 'modest map display')
        self.lat = None
        self.lon = None
        self.alt = None
        self.speed = None
        self.airspeed = None
        self.groundspeed = None
        self.heading = 0
        self.wp_change_time = 0
        self.fence_change_time = 0
        self.server = None
        self.server = mmap_server.start_server('127.0.0.1', port=9999, module_state=self)
        webbrowser.open('http://127.0.0.1:9999/', autoraise=True)

    def unload(self):
        """unload module"""
        self.server.terminate()

    def mavlink_packet(self, m):
        """handle an incoming mavlink packet"""
        mtype = m.get_type()
        if mtype == 'GPS_RAW':
            (self.lat, self.lon) = (m.lat, m.lon)
        elif mtype == 'GPS_RAW_INT':
            (self.lat, self.lon) = (m.lat / 1.0e7, m.lon / 1.0e7)
        elif mtype == "VFR_HUD":
            self.heading = m.heading
            self.alt = m.alt
            self.airspeed = m.airspeed
            self.groundspeed = m.groundspeed

def init(mpstate):
    '''initialise module'''
    return MMapModule(mpstate)


# Copyright (C) 2016  Intel Corporation. All rights reserved.
#
# This file is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Compass calibration helper. This is adapted from Ardupilot's local MAVProxy
module magcal_graph. Some people read it "magical_graph" at the time of
release, thus the silly reason for the name of this module.
"""
from __future__ import print_function

import sys

from pymavlink import mavutil

from MAVProxy.modules.lib import mp_module, mp_util, multiproc

try:
    import pkg_resources
    datapath = pkg_resources.resource_filename(__name__, 'data')
except:
    import os.path as p
    datapath = p.join(p.dirname(p.abspath(__file__)), 'data')

class MagicalModule(mp_module.MPModule):
    def __init__(self, mpstate):
        super(MagicalModule, self).__init__(mpstate, 'magical')
        self.add_command(
            'magical_ui',
            self.cmd_magical_ui,
            'open the GUI for compass calibration',
        )

        self.mpstate = mpstate
        self.parent_pipe, self.child_pipe = multiproc.Pipe()
        self.ui_process = None
        self.progress_msgs = {}
        self.report_msgs = {}
        self.attitude_msg = None
        self.raw_imu_msg = None
        self.running = False
        self.last_ui_msgs = {}

    def start_ui(self):
        if self.ui_is_active():
            return

        self.ui_process = multiproc.Process(target=self.ui_task)
        self.update_ui()
        self.ui_process.start()

    def stop_ui(self):
        if not self.ui_is_active():
            return

        self.parent_pipe.send(dict(name='close'))
        self.ui_process.join(2)
        if self.ui_process.is_alive():
            print("magical: UI process timed out, killing it", file=sys.stderr)
            self.kill_ui()

    def kill_ui(self):
        self.ui_process.terminate()
        self.parent_pipe, self.child_pipe = multiproc.Pipe()

    def ui_task(self):
        mp_util.child_close_fds()

        from MAVProxy.modules.lib import wx_processguard
        from MAVProxy.modules.lib.wx_loader import wx
        from MAVProxy.modules.mavproxy_magical.magical_ui import MagicalFrame

        app = wx.App(False)
        app.frame = MagicalFrame(self.child_pipe)
        app.frame.Show()
        app.MainLoop()

    def process_ui_commands(self):
        while self.parent_pipe.poll():
            cmd = self.parent_pipe.recv()

            if cmd == 'start':
                self.mpstate.functions.process_stdin('magcal start')
            elif cmd == 'cancel':
                self.mpstate.functions.process_stdin('magcal cancel')

    def ui_is_active(self):
        return self.ui_process is not None and self.ui_process.is_alive()

    def idle_task(self):
        self.process_ui_commands()
        if self.ui_is_active():
            self.update_ui()
        elif self.last_ui_msgs:
            self.last_ui_msgs = {}

    def cmd_magical_ui(self, args):
        self.start_ui()

    def mavlink_packet(self, m):
        t = m.get_type()
        if t == 'MAG_CAL_PROGRESS':
            self.progress_msgs[m.compass_id] = m
            if m.compass_id in self.report_msgs:
                del self.report_msgs[m.compass_id]
            if 'report' in self.last_ui_msgs:
                del self.last_ui_msgs['report']
            self.running = True
        elif t == 'MAG_CAL_REPORT':
            self.report_msgs[m.compass_id] = m
            if self.report_msgs and len(self.report_msgs) == len(self.progress_msgs):
                self.running = False
        elif t == 'ATTITUDE':
            self.attitude_msg = m
        elif t == 'RAW_IMU':
            self.raw_imu_msg = m
        elif t == 'COMMAND_ACK':
            if m.command == mavutil.mavlink.MAV_CMD_DO_CANCEL_MAG_CAL:
                self.running = False

    def unload(self):
        self.stop_ui()

    def send_ui_msg(self, m):
        send = False
        if m['name'] not in self.last_ui_msgs:
            send = True
        else:
            last = self.last_ui_msgs[m['name']]
            if m['name'] == 'report':
                if len(m['messages']) != len(last['messages']):
                    send = True
                else:
                    for a, b in zip(m['messages'], last['messages']):
                        if a.compass_id != b.compass_id:
                            send = True
                            break
            else:
                send = m != last

        if not send:
            return

        self.parent_pipe.send(m)
        self.last_ui_msgs[m['name']] = m

    def update_ui(self):
        self.send_ui_msg(dict(
            name='ui_is_active',
            value=self.ui_is_active(),
        ))

        if self.progress_msgs:
            pct = min(p.completion_pct for p in self.progress_msgs.values())
            masks = [~0] * 10
            for p in self.progress_msgs.values():
                for i in range(10):
                    masks[i] &= p.completion_mask[i]
            visible = [bool(m & 1 << j) for m in masks for j in range(8)]

            self.send_ui_msg(dict(
                name='progress_update',
                pct=pct,
                sections=visible,
            ))

        if self.report_msgs and len(self.report_msgs) == len(self.progress_msgs):
            keys = sorted(self.progress_msgs.keys())
            self.send_ui_msg(dict(
                name='report',
                messages=[self.report_msgs[k] for k in keys],
            ))

        if self.attitude_msg:
            self.send_ui_msg(dict(
                name='attitude',
                roll=self.attitude_msg.roll,
                pitch=self.attitude_msg.pitch,
                yaw=self.attitude_msg.yaw,
                timestamp=self.attitude_msg.time_boot_ms,
            ))

        if self.raw_imu_msg:
            self.send_ui_msg(dict(
                name='mag',
                x=self.raw_imu_msg.xmag,
                y=self.raw_imu_msg.ymag,
                z=self.raw_imu_msg.zmag,
            ))


        self.send_ui_msg(dict(
            name='running',
            value=self.running,
        ))

def init(mpstate):
    return MagicalModule(mpstate)



#!/usr/bin/env python
'''
System_Time Module
Peter barker, May 2018

Send mavlink SYSTEM_TIME messages via mavlink

'''

import time

from MAVProxy.modules.lib import mp_module
from MAVProxy.modules.lib import mp_settings


class system_time(mp_module.MPModule):
    def __init__(self, mpstate):
        """Initialise module"""
        super(system_time, self).__init__(mpstate, "system_time", "")
        self.last_sent = 0
        self.last_sent_ts1 = 0
        self.last_sent_timesync = 0
        self.module_load_time = time.time()

        self.system_time_settings = mp_settings.MPSettings(
            [('verbose', bool, False),
             ('interval_timesync', int, 10),
             ('interval', int, 10)])
        self.add_command('system_time',
                         self.cmd_system_time,
                         "system_time module",
                         ['status', 'set (LOGSETTING)'])

    def usage(self):
        '''show help on command line options'''
        return "Usage: system_time <status|set>"

    def cmd_system_time(self, args):
        '''control behaviour of the module'''
        if len(args) == 0:
            print(self.usage())
        elif args[0] == "status":
            print(self.status())
        elif args[0] == "set":
            self.system_time_settings.command(args[1:])
        else:
            print(self.usage())

    def status(self):
        '''returns information about module'''
        return "Perfectly happy.  All is good with the world"

    def uptime(self):
        '''return system uptime in ms'''
        return int(1000*(time.time()-self.module_load_time))

    def idle_task(self):
        '''called rapidly by mavproxy'''
        now = time.time()

        if now-self.last_sent > self.system_time_settings.interval:
            self.last_sent = now
            time_us = time.time() * 1000000
            if self.system_time_settings.verbose:
                print("ST: Sending system time: (%u/%u)" %
                      (time_us, self.uptime(),))
            self.master.mav.system_time_send(int(time_us),
                                             self.uptime())

        if (now-self.last_sent_timesync >
            self.system_time_settings.interval_timesync):
            self.last_sent_timesync = now
            time_ns = time.time() * 1000000000
            time_ns += 1234
            if self.system_time_settings.verbose:
                print("ST: Sending timesync request")
            self.master.mav.timesync_send(0, int(time_ns))
            self.last_sent_ts1 = time_ns

    def mavlink_packet(self, m):
        '''handle mavlink packets'''
        if m.get_type() == 'SYSTEM_TIME':
            if self.system_time_settings.verbose:
                print("ST: Received from (%u/%u): %s" %
                      (m.get_srcSystem(), m.get_srcComponent(), m))
        if m.get_type() == 'TIMESYNC':
            if m.tc1 == 0:
                # this is a request for a timesync response
                time_ns = time.time() * 1000000000
                time_ns += 1234
                if True or self.system_time_settings.verbose:
                    if self.system_time_settings.verbose:
                        print("ST: received timesync; sending response: %u" %
                              (time_ns))
                    self.master.mav.timesync_send(int(time_ns),
                                                  m.ts1)
            else:
                if m.ts1 == self.last_sent_ts1:
                    # we sent this one!
                    now_ns = time.time() * 1000000000
                    now_ns += 1234
                    if self.system_time_settings.verbose:
                        print("ST: timesync response: sysid=%u latency=%fms" %
                              (m.get_srcSystem(),
                               (now_ns-self.last_sent_ts1)/1000000.0))

def init(mpstate):
    '''initialise module'''
    return system_time(mpstate)



#!/usr/bin/env python
'''
Example Module
Peter Barker, September 2016

This module simply serves as a starting point for your own MAVProxy module.

1. copy this module sidewise (e.g. "cp mavproxy_example.py mavproxy_coolfeature.py"
2. replace all instances of "example" with whatever your module should be called
(e.g. "coolfeature")

3. trim (or comment) out any functionality you do not need
'''

import os
import os.path
import sys
from pymavlink import mavutil
import errno
import time

from MAVProxy.modules.lib import mp_module
from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import mp_settings


class example(mp_module.MPModule):
    def __init__(self, mpstate):
        """Initialise module"""
        super(example, self).__init__(mpstate, "example", "")
        self.status_callcount = 0
        self.boredom_interval = 10 # seconds
        self.last_bored = time.time()

        self.packets_mytarget = 0
        self.packets_othertarget = 0

        self.example_settings = mp_settings.MPSettings(
            [ ('verbose', bool, False),
          ])
        self.add_command('example', self.cmd_example, "example module", ['status','set (LOGSETTING)'])

    def usage(self):
        '''show help on command line options'''
        return "Usage: example <status|set>"

    def cmd_example(self, args):
        '''control behaviour of the module'''
        if len(args) == 0:
            print(self.usage())
        elif args[0] == "status":
            print(self.status())
        elif args[0] == "set":
            self.example_settings.command(args[1:])
        else:
            print(self.usage())

    def status(self):
        '''returns information about module'''
        self.status_callcount += 1
        self.last_bored = time.time() # status entertains us
        return("status called %(status_callcount)d times.  My target positions=%(packets_mytarget)u  Other target positions=%(packets_mytarget)u" %
               {"status_callcount": self.status_callcount,
                "packets_mytarget": self.packets_mytarget,
                "packets_othertarget": self.packets_othertarget,
               })

    def boredom_message(self):
        if self.example_settings.verbose:
            return ("I'm very bored")
        return ("I'm bored")

    def idle_task(self):
        '''called rapidly by mavproxy'''
        now = time.time()
        if now-self.last_bored > self.boredom_interval:
            self.last_bored = now
            message = self.boredom_message()
            self.say("%s: %s" % (self.name,message))
            # See if whatever we're connected to would like to play:
            self.master.mav.statustext_send(mavutil.mavlink.MAV_SEVERITY_NOTICE,
                                            message)

    def mavlink_packet(self, m):
        '''handle mavlink packets'''
        if m.get_type() == 'GLOBAL_POSITION_INT':
            if self.settings.target_system == 0 or self.settings.target_system == m.get_srcSystem():
                self.packets_mytarget += 1
            else:
                self.packets_othertarget += 1

def init(mpstate):
    '''initialise module'''
    return example(mpstate)



#!/usr/bin/env python

"""
  MAVProxy checklist, implemented in a child process
  Created by Stephen Dade (stephen_dade@hotmail.com)
"""

import sys
from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import multiproc
from MAVProxy.modules.lib.wx_loader import wx

class CheckItem():
    '''Checklist item used for information transfer
    between threads/processes/pipes'''
    def __init__(self, name, state):
        self.name = name
        self.state = state


class CheckUI():
    '''
    a checklist UI for MAVProxy
    '''
    def __init__(self, title='MAVProxy: Checklist'):
        import threading
        self.title  = title
        self.menu_callback = None
        self.parent_pipe,self.child_pipe = multiproc.Pipe()
        self.close_event = multiproc.Event()
        self.close_event.clear()
        self.child = multiproc.Process(target=self.child_task)
        self.child.start()

    def child_task(self):
        '''child process - this holds all the GUI elements'''
        mp_util.child_close_fds()
        from MAVProxy.modules.lib.wx_loader import wx

        app = wx.App(False)
        app.frame = ChecklistFrame(state=self, title=self.title)
        app.frame.Show()
        app.MainLoop()

    def close(self):
        '''close the UI'''
        self.close_event.set()
        if self.is_alive():
            self.child.join(2)

    def is_alive(self):
        '''check if child is still going'''
        return self.child.is_alive()

    def set_check(self, name, state):
        '''set a status value'''
        if self.child.is_alive():
            self.parent_pipe.send(CheckItem(name, state))


class ChecklistFrame(wx.Frame):
    """ The main frame of the console"""

    def __init__(self, state, title):
        self.state = state
        wx.Frame.__init__(self, None, title=title, size=(350,400), style=wx.DEFAULT_FRAME_STYLE ^ wx.RESIZE_BORDER)

        #use tabs for the individual checklists
        self.createLists()
        self.panel = wx.Panel(self)
        self.nb = wx.Choicebook(self.panel, wx.ID_ANY)

        #create the tabs
        self.createWidgets()

        #assign events to the buttons on the tabs
        self.createActions()

        #add in the pipe from MAVProxy
        self.timer = wx.Timer(self)
        #self.Bind(wx.EVT_TIMER, self.on_timer, self.timer)
        self.Bind(wx.EVT_TIMER, lambda evt, notebook=self.nb: self.on_timer(evt, notebook), self.timer)
        self.timer.Start(100)

        # finally, put the notebook in a sizer for the panel to manage
        # the layout
        sizer = wx.BoxSizer()
        sizer.Add(self.nb, 1, wx.EXPAND)
        self.panel.SetSizer(sizer)

        self.Show(True)
        self.pending = []

    #Create the checklist items
    def createLists(self):
        '''Generate the checklists. Note that:
        0,1 = off/on for auto-ticked items
        2,3 = off/on for manually ticked items'''

        self.beforeAssemblyList = {
        'Confirm batteries charged':2,
        'No physical damage to airframe':2,
        'All electronics present and connected':2,
        'Bottle loaded':2,
        'Ground station operational':2
        }

        self.beforeEngineList = {
        'Avionics Power ON':2,
        'Pixhawk Booted':0,
        'Odroid Booted':2,
        'Cameras calibrated and capturing':2,
        'GPS lock':0,
        'Airspeed check':2,
        'Barometer check':2,
        'Compass check':2,
        'Flight mode MANUAL':0,
        'Avionics Power':0,
        'Servo Power':0,
        'IMU Check':0,
        'Aircraft Params Loaded':2,
        'Waypoints Loaded':0,
        'Servo and clevis check':2,
        'Geofence loaded':2,
        'Ignition circuit and battery check':2,
        'Check stabilisation in FBWA mode':2
        }

        self.beforeTakeoffList = {
        'Engine throttle responsive':2,
        'Runway clear':2,
        'Radio links > 6db margin':0,
        'Antenna tracker check':2,
        'GCS stable':2,
        }

        self.beforeCruiseList = {
        'Airspeed > 10 m/s':0,
        'Altitude > 30 m':0,
        '< 100 degrees to 1st Waypoint':2,
        'Airspeed and climb rate nominal':2
        }

        self.bottleDropList = {
        'Joe found':2,
        'Joe waypoint laid in':2,
        '< 100m to Joe waypoint':2,
        'Bottle drop mechanism activated':2
        }

        self.beforeLandingList = {
        'Runway clear':2,
        'APM set to FBWA mode':2,
        '< 100m from airfield home':2
        }

        self.beforeShutdownList = {
        'Taxi to parking':2,
        'Engine cutoff':2,
        'Data downloaded':2,
        'Ignition power off':2,
        'Avionics power off':2
        }

    # create controls on form - labels, buttons, etc
    def createWidgets(self):
        #create the panels for the tabs
        PanelAssembly = wx.Panel(self.nb)
        boxAssembly = wx.BoxSizer(wx.VERTICAL)
        PanelAssembly.SetAutoLayout(True)
        PanelAssembly.SetSizer(boxAssembly)
        PanelAssembly.Layout()

        PanelEngine = wx.Panel(self.nb)
        boxEngine = wx.BoxSizer(wx.VERTICAL)
        PanelEngine.SetAutoLayout(True)
        PanelEngine.SetSizer(boxEngine)
        PanelEngine.Layout()

        PanelTakeoff = wx.Panel(self.nb)
        boxTakeoff = wx.BoxSizer(wx.VERTICAL)
        PanelTakeoff.SetAutoLayout(True)
        PanelTakeoff.SetSizer(boxTakeoff)
        PanelTakeoff.Layout()

        PanelCruise = wx.Panel(self.nb)
        boxCruise = wx.BoxSizer(wx.VERTICAL)
        PanelCruise.SetAutoLayout(True)
        PanelCruise.SetSizer(boxCruise)
        PanelCruise.Layout()

        PanelDrop = wx.Panel(self.nb)
        boxDrop = wx.BoxSizer(wx.VERTICAL)
        PanelDrop.SetAutoLayout(True)
        PanelDrop.SetSizer(boxDrop)
        PanelDrop.Layout()

        PanelLanding = wx.Panel(self.nb)
        boxLanding = wx.BoxSizer(wx.VERTICAL)
        PanelLanding.SetAutoLayout(True)
        PanelLanding.SetSizer(boxLanding)
        PanelLanding.Layout()

        PanelShutdown = wx.Panel(self.nb)
        boxShutdown = wx.BoxSizer(wx.VERTICAL)
        PanelShutdown.SetAutoLayout(True)
        PanelShutdown.SetSizer(boxShutdown)
        PanelShutdown.Layout()

        #add the data to the individual tabs

        '''before assembly checklist'''
        for key in self.beforeAssemblyList:
            if self.beforeAssemblyList[key] == 0:
                disCheckBox = wx.CheckBox(PanelAssembly, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxAssembly.Add(disCheckBox)
            if self.beforeAssemblyList[key] == 2:
                boxAssembly.Add(wx.CheckBox(PanelAssembly, wx.ID_ANY, key))

        self.AssemblyButton = wx.Button(PanelAssembly, wx.ID_ANY, "Close final hatches")
        boxAssembly.Add(self.AssemblyButton)

        '''before Engine Start checklist'''
        for key in self.beforeEngineList:
            if self.beforeEngineList[key] == 0:
                disCheckBox = wx.CheckBox(PanelEngine, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxEngine.Add(disCheckBox)
            if self.beforeEngineList[key] == 2:
                boxEngine.Add(wx.CheckBox(PanelEngine, wx.ID_ANY, key))

        self.EngineButton = wx.Button(PanelEngine, wx.ID_ANY, "Ready for Engine start")
        boxEngine.Add(self.EngineButton)

        '''before takeoff checklist'''
        for key in self.beforeTakeoffList:
            if self.beforeTakeoffList[key] == 0:
                disCheckBox = wx.CheckBox(PanelTakeoff, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxTakeoff.Add(disCheckBox)
            if self.beforeTakeoffList[key] == 2:
                boxTakeoff.Add(wx.CheckBox(PanelTakeoff, wx.ID_ANY, key))

        self.TakeoffButton = wx.Button(PanelTakeoff, wx.ID_ANY, "Ready for Takeoff")
        boxTakeoff.Add(self.TakeoffButton)

        '''before cruise/AUTO checklist'''
        for key in self.beforeCruiseList:
            if self.beforeCruiseList[key] == 0:
                disCheckBox = wx.CheckBox(PanelCruise, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxCruise.Add(disCheckBox)
            if self.beforeCruiseList[key] == 2:
                boxCruise.Add(wx.CheckBox(PanelCruise, wx.ID_ANY, key))

        self.CruiseButton = wx.Button(PanelCruise, wx.ID_ANY, "Ready for Cruise/AUTO")
        boxCruise.Add(self.CruiseButton)

        '''before bottle drop checklist'''
        for key in self.bottleDropList:
            if self.bottleDropList[key] == 0:
                disCheckBox = wx.CheckBox(PanelDrop, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxDrop.Add(disCheckBox)
            if self.bottleDropList[key] == 2:
                boxDrop.Add(wx.CheckBox(PanelDrop, wx.ID_ANY, key))

        self.DropButton = wx.Button(PanelDrop, wx.ID_ANY, "Ready for Bottle Drop")
        boxDrop.Add(self.DropButton)

        '''before landing checklist'''
        for key in self.beforeLandingList:
            if self.beforeLandingList[key] == 0:
                disCheckBox = wx.CheckBox(PanelLanding, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxLanding.Add(disCheckBox)
            if self.beforeLandingList[key] == 2:
                boxLanding.Add(wx.CheckBox(PanelLanding, wx.ID_ANY, key))

        self.LandingButton = wx.Button(PanelLanding, wx.ID_ANY, "Ready for Landing")
        boxLanding.Add(self.LandingButton)

        '''before shutdown checklist'''
        for key in self.beforeShutdownList:
            if self.beforeShutdownList[key] == 0:
                disCheckBox = wx.CheckBox(PanelShutdown, wx.ID_ANY, key)
                disCheckBox.Enable(False)
                boxShutdown.Add(disCheckBox)
            if self.beforeShutdownList[key] == 2:
                boxShutdown.Add(wx.CheckBox(PanelShutdown, wx.ID_ANY, key))

        self.ShutdownButton = wx.Button(PanelShutdown, wx.ID_ANY, "Ready for Shutdown")
        boxShutdown.Add(self.ShutdownButton)

        #and add in the tabs
        self.nb.AddPage(PanelAssembly, "1. During Assembly")
        self.nb.AddPage(PanelEngine, "2. Before Engine Start")
        self.nb.AddPage(PanelTakeoff, "3. Before Takeoff")
        self.nb.AddPage(PanelCruise, "4. Before Cruise/AUTO")
        self.nb.AddPage(PanelDrop, "5. Before Bottle Drop")
        self.nb.AddPage(PanelLanding, "6. Before Landing")
        self.nb.AddPage(PanelShutdown, "7. Before Shutdown")

    #Create the actions for the buttons
    def createActions(self):
        self.Bind(wx.EVT_BUTTON, self.on_Button, self.AssemblyButton)
        self.Bind(wx.EVT_BUTTON, self.on_Button, self.EngineButton)
        self.Bind(wx.EVT_BUTTON, self.on_Button, self.TakeoffButton)
        self.Bind(wx.EVT_BUTTON, self.on_Button, self.CruiseButton)
        self.Bind(wx.EVT_BUTTON, self.on_Button, self.DropButton)
        self.Bind(wx.EVT_BUTTON, self.on_Button, self.LandingButton)
        self.Bind(wx.EVT_BUTTON, self.on_ButtonLast, self.ShutdownButton)

    #do a final check of the current panel and move to the next
    def on_Button( self, event ):
        win = (event.GetEventObject()).GetParent()
        for widget in win.GetChildren():
            if type(widget) is wx.CheckBox and widget.IsChecked() == 0:
                dlg = wx.MessageDialog(win, "Not all items checked", "Error", wx.OK | wx.ICON_WARNING)
                dlg.ShowModal()
                dlg.Destroy()
                return
        #all checked, go to next panel.
        win.GetParent().AdvanceSelection()

    #Special implementation of the above function, but for the last tab
    def on_ButtonLast( self, event ):
        win = (event.GetEventObject()).GetParent()
        for widget in win.GetChildren():
            if type(widget) is wx.CheckBox and widget.IsChecked() == 0:
                dlg = wx.MessageDialog(win, "Not all items checked", "Error", wx.OK | wx.ICON_WARNING)
                dlg.ShowModal()
                dlg.Destroy()
                return
        #all checked, we're done.
        dlg = wx.MessageDialog(win, "Checklist Complete", "Done", wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    #Receive messages from MAVProxy and process them
    def on_timer(self, event, notebook):
        state = self.state
        win = notebook.GetPage(notebook.GetSelection())
        if state.close_event.wait(0.001):
            self.timer.Stop()
            self.Destroy()
            return
        while state.child_pipe.poll():
            obj = state.child_pipe.recv()
            if isinstance(obj, CheckItem):
                #go through each item in the current tab and (un)check as needed
                #print(obj.name + ", " + str(obj.state))
                for widget in win.GetChildren():
                    if type(widget) is wx.CheckBox and widget.GetLabel() == obj.name:
                        widget.SetValue(obj.state)


if __name__ == "__main__":
    # test the console
    import time

    checklist = CheckUI()

    #example auto-tick in second tab page
    while checklist.is_alive():
        checklist.set_check("Compass Calibrated", 1)
        time.sleep(0.5)



#!/usr/bin/env python

from __future__ import print_function

'''firmware handling'''

import time, os, fnmatch
import json
import threading
import re
import subprocess

from pymavlink import mavutil, mavparm
from MAVProxy.modules.lib import mp_util
from MAVProxy.modules.lib import mp_module
from MAVProxy.modules.lib import multiproc
from MAVProxy.modules.lib import mp_settings

class FirmwareModule(mp_module.MPModule):

    def __init__(self, mpstate):
        super(FirmwareModule, self).__init__(mpstate, "firmware", "firmware handling", public = True)
        self.firmware_settings = mp_settings.MPSettings(
            [('uploader', str, "uploader.py"),
            ])
        self.add_command('fw', self.cmd_fw, "firmware handling",
                         ["<manifest> (OPT)",
                          "list <filterterm...>",
                          "flash <filterterm...>",
                          "set (FIRMWARESETTING)",
                          "flashfile FILENAME"])
        self.add_completion_function('(FIRMWARESETTING)',
                                     self.firmware_settings.completion)
        self.downloaders_lock = threading.Lock()
        self.downloaders = {}
        self.manifests_parse()

    def usage(self):
        '''show help on a command line options'''
        return '''Usage: fw <manifest|set|list|flash|flashfile>

# Use the column headings from "list" as filter terms

# e.g. list latest releases for PX4-v2 platform:
fw list latest=1 platform=PX4-v2

# e.g. download most recent official PX4-v2 release for quadcopters:
fw download releasetype=OFFICIAL frame=quad platform=PX4-v2
'''

    def cmd_fw_help(self):
        '''show help on fw command'''
        print(self.usage())

    def cmd_fw(self, args):
        '''execute command defined in args'''
        if len(args) == 0:
            print(self.usage())
            return
        rest = args[1:]
        if args[0] == "manifest":
            self.cmd_fw_manifest(rest)
        elif args[0] == "list":
            self.cmd_fw_list(rest)
        elif args[0] == "flash":
            self.cmd_flash(rest)
        elif args[0] == "set":
            self.firmware_settings.command(args[1:])
        elif args[0] == "download":
            self.cmd_fw_download(rest)
        elif args[0] in ["help","usage"]:
            self.cmd_fw_help(rest)
        elif args[0] == "flashfile":
            self.cmd_flashfile(rest)
        else:
            print(self.usage())

    def frame_from_firmware(self, firmware):
        '''extract information from firmware, return pretty string to user'''
        # see Tools/scripts/generate-manifest for this map:
        frame_to_mavlink_dict = {
            "quad": "QUADROTOR",
            "hexa": "HEXAROTOR",
            "y6": "ARDUPILOT_Y6",
            "tri": "TRICOPTER",
            "octa": "OCTOROTOR",
            "octa-quad": "ARDUPILOT_OCTAQUAD",
            "heli": "HELICOPTER",
            "Plane": "FIXED_WING",
            "Tracker": "ANTENNA_TRACKER",
            "Rover": "GROUND_ROVER",
            "PX4IO": "ARDUPILOT_PX4IO",
        }
        mavlink_to_frame_dict = { v : k  for k,v in frame_to_mavlink_dict.items() }
        x = firmware["mav-type"]
        if firmware["mav-autopilot"] != "ARDUPILOTMEGA":
            return x
        if x in mavlink_to_frame_dict:
            return mavlink_to_frame_dict[x]

        return x

    def semver_from_firmware(self, firmware):
        '''Extract a tuple of (major,minor,patch) from a firmware instance'''
        version = firmware.get("mav-firmware-version-major",None)
        if version is None:
            return (None,None,None)
        return (firmware["mav-firmware-version-major"],
                firmware["mav-firmware-version-minor"],
                firmware["mav-firmware-version-patch"])

    def row_is_filtered(self, row_subs, filters):
        '''returns True if row should NOT be included according to filters'''
        for filtername in filters:
            filtervalue = filters[filtername]
            if filtername in row_subs:
                row_subs_value = row_subs[filtername]
                if str(row_subs_value) != str(filtervalue):
                    return True
            else:
                print("fw: Unknown filter keyword (%s)" % (filtername,))
        return False

    def filters_from_args(self, args):
        '''take any argument of the form name=value anmd put it into a dict; return that and the remaining arguments'''
        filters = dict()
        remainder = []
        for arg in args:
            try:
                equals = arg.index('=')
                # anything ofthe form key-value is taken as a filter
                filters[arg[0:equals]] = arg[equals+1:];
            except ValueError:
                remainder.append(arg)
        return (filters,remainder)

    def all_firmwares(self):
        ''' return firmware entries from all manifests'''
        all = []
        for manifest in self.manifests:
            for firmware in manifest["firmware"]:
                all.append(firmware)
        return all

    def rows_for_firmwares(self, firmwares):
        '''provide user-readable text for a firmware entry'''
        rows = []
        i = 0
        for firmware in firmwares:
            frame = self.frame_from_firmware(firmware)
            row = {
                "seq": i,
                "platform": firmware["platform"],
                "frame": frame,
#                "type": firmware["mav-type"],
                "releasetype": firmware["mav-firmware-version-type"],
                "latest": firmware["latest"],
                "git-sha": firmware["git-sha"][0:7],
                "format": firmware["format"],
                "_firmware": firmware,
            }
            (major,minor,patch) = self.semver_from_firmware(firmware)
            if major is None:
                row["version"] = ""
                row["major"] = ""
                row["minor"] = ""
                row["patch"] = ""
            else:
                row["version"] = firmware["mav-firmware-version"]
                row["major"] = major
                row["minor"] = minor
                row["patch"] = patch

            i += 1
            rows.append(row)

        return rows

    def filter_rows(self, filters, rows):
        '''returns rows as filtered by filters'''
        ret = []
        for row in rows:
            if not self.row_is_filtered(row, filters):
                ret.append(row)
        return ret

    def filtered_rows_from_args(self, args):
        '''extracts filters from args, rows from manifests, returns filtered rows'''
        if len(self.manifests) == 0:
            print("fw: No manifests downloaded.  Try 'fw manifest download'")
            return None

        (filters,remainder) = self.filters_from_args(args)
        all = self.all_firmwares()
        rows = self.rows_for_firmwares(all)
        filtered = self.filter_rows(filters, rows)
        return (filtered, remainder)

    def cmd_flashfile(self, args):
        filename = args[0]
        subprocess.check_call([self.firmware_settings.uploader, filename])

    def cmd_flash(self, args):
        '''cmd handler for flash'''
        stuff = self.filtered_rows_from_args(args)
        (filtered, remainder) = stuff
        if stuff is None:
            print("Nothing returned from filter")
            return

        (filtered, remainder) = stuff

        if len(filtered) != 1:
            print("Filter must return single firmware")
            self.list_firmwares(filtered)
            print("Filter must return single firmware")
            return

        firmware = filtered[0]["_firmware"]

        try:
            filename = self.download_firmware(firmware)
        except Exception as e:
            print("fw: download failed")
            print(e)
            return

        subprocess.check_call([self.firmware_settings.uploader, filename])
        return


    def cmd_fw_list(self, args):
        '''cmd handler for list'''
        stuff = self.filtered_rows_from_args(args)
        if stuff is None:
            return
        (filtered, remainder) = stuff
        self.list_firmwares(filtered)

    def list_firmwares(self, filtered):
        print("")
        print(" seq platform frame    major.minor.patch releasetype latest git-sha format")
        for row in filtered:
            print("{seq:>5} {platform:<13} {frame:<10} {version:<10} {releasetype:<9} {latest:<6} {git-sha} {format}".format(**row))
        print(" seq platform frame    major.minor.patch releasetype latest git-sha format")

    def cmd_fw_download(self, args):
        '''cmd handler for downloading firmware'''
        stuff = self.filtered_rows_from_args(args)
        if stuff is None:
            return
        (filtered, remainder) = stuff
        if len(filtered) == 0:
            print("fw: No firmware specified")
            return
        if len(filtered) > 1:
            print("fw: No single firmware specified")
            return

        try:
            self.download_firmware(filtered[0]["_firmware"])
        except Exception as e:
            print("fw: download failed")
            print(e)

    def download_firmware(self, firmware, filename=None):
        url = firmware["url"]

        print("fw: URL: %s"  % (url,))
        if filename is None:
            filename = os.path.basename(url)
        files = []
        files.append((url,filename))
        child = multiproc.Process(target=mp_util.download_files, args=(files,))
        child.start()
        tstart = time.time()
        while True:
            if time.time() - tstart > 60:
                print("Download timeout")
            if not child.is_alive():
                break
            print("Waiting for download to compliete...")
            time.sleep(1)

        return filename

    def fw_manifest_usage(self):
        '''return help on manifest subcommand'''
        return("Usage: fw manifest <list|download|purge>")

    def cmd_fw_manifest_help(self):
        '''show help on manifest subcommand'''
        print(self.fw_manifest_usage())

    def find_manifests(self):
        '''locate manifests and return filepaths thereof'''
        manifest_dir = mp_util.dot_mavproxy()

        ret = []
        for file in os.listdir(manifest_dir):
            try:
                file.index("manifest")
                ret.append(os.path.join(manifest_dir,file))
            except ValueError:
                pass
        return ret

    def cmd_fw_manifest_list(self):
        '''list manifests'''
        for filepath in self.find_manifests():
            print(filepath)

    def cmd_fw_manifest_load(self):
        '''handle command to load manifests - hidden command since these should only change with fw commands'''
        self.manifests_parse()

    def cmd_fw_manifest_purge(self):
        '''remove all downloaded manifests'''
        for filepath in self.find_manifests():
            os.unlink(filepath)
        self.manifests_parse()

    def cmd_fw_manifest(self, args):
        '''cmd handler for manipulating manifests'''
        if len(args) == 0:
            print(self.fw_manifest_usage())
            return
        rest = args[1:]
        if args[0] == "download":
            return self.manifest_download()
        if args[0] == "list":
            return self.cmd_fw_manifest_list()
        if args[0] == "load":
            return self.cmd_fw_manifest_load()
        if args[0] == "purge":
            return self.cmd_fw_manifest_purge()
        if args[0] == "help":
            return self.cmd_fw_manifest_help()
        else:
            print("fw: Unknown manifest option (%s)" % args[0])
            print(fw_manifest_usage())

    def manifest_parse(self, path):
        '''parse manifest at path, return JSON object'''
        print("fw: parsing manifests")
        content = open(path).read()
        return json.loads(content)

    def semver_major(self,semver):
        '''return major part of semver version number. Avoids "import semver"'''
        return int(semver[0:semver.index(".")])

    def manifest_path_is_old(self, path):
        '''return true if path is more than a day old'''
        mtime = os.path.getmtime(path)
        return (time.time() - mtime) > 24*60*60

    def manifests_parse(self):
        '''parse manifests present on system'''
        self.manifests = []
        for manifest_path in self.find_manifests():
            if self.manifest_path_is_old(manifest_path):
                print("fw: Manifest (%s) is old; consider 'fw manifest download'" % (manifest_path))
            manifest = self.manifest_parse(manifest_path)
            if self.semver_major(manifest["format-version"]) != 1:
                print("fw: Manifest (%s) has major version %d; MAVProxy only understands version 1" % (manifest_path,manifest["format-version"]))
                continue
            self.manifests.append(manifest)

    def download_url(self, url, path):
        mp_util.download_files([(url,path)])

    def idle_task(self):
        '''called rapidly by mavproxy'''
        if self.downloaders_lock.acquire(False):
            removed_one = False
            for url in self.downloaders.keys():
                if not self.downloaders[url].is_alive():
                    print("fw: Download thread for (%s) done" % url)
                    del self.downloaders[url]
                    removed_one = True
            if removed_one and not self.downloaders.keys():
                # all downloads finished - parse them
                self.manifests_parse()

            self.downloaders_lock.release()

    def make_safe_filename_from_url(self, url):
        '''return a version of url safe for use as a filename'''
        r = re.compile("([^a-zA-Z0-9_.-])")
        filename = r.sub(lambda m : "%" + str(hex(ord(str(m.group(1))))).upper(), url)
        return filename

    def manifest_download(self):
        '''download manifest files'''
        if self.downloaders_lock.acquire(False):
            if len(self.downloaders):
                # there already exist downloader threads
                self.downloaders_lock.release()
                return

            for url in ['https://firmware.ardupilot.org/manifest.json']:
                filename = self.make_safe_filename_from_url(url)
                path = mp_util.dot_mavproxy("manifest-%s" % filename)
                self.downloaders[url] = threading.Thread(target=self.download_url, args=(url, path))
                self.downloaders[url].start()
            self.downloaders_lock.release()
        else:
            print("fw: Failed to acquire download lock")

def init(mpstate):
    '''initialise module'''
    return FirmwareModule(mpstate)



"""
  MAVProxy realtime graphing module

  uses lib/live_graph.py for display
"""

from pymavlink import mavutil
import re, os, sys

from MAVProxy.modules.lib import live_graph

from MAVProxy.modules.lib import mp_module

class GraphModule(mp_module.MPModule):
    def __init__(self, mpstate):
        super(GraphModule, self).__init__(mpstate, "graph", "graph control")
        self.timespan = 20
        self.tickresolution = 0.2
        self.graphs = []
        self.add_command('graph', self.cmd_graph, "[expression...] add a live graph",
                         ['(VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE)',
                          'legend',
                          'timespan',
                          'tickresolution'])
        self.legend = {
        }
        

    def cmd_graph(self, args):
        '''graph command'''
        if len(args) == 0:
            # list current graphs
            for i in range(len(self.graphs)):
                print("Graph %u: %s" % (i, self.graphs[i].fields))
            return

        elif args[0] == "help":
            print("graph <timespan|tickresolution|expression>")
        elif args[0] == "timespan":
            if len(args) == 1:
                print("timespan: %.1f" % self.timespan)
                return
            self.timespan = float(args[1])
        elif args[0] == "tickresolution":
            if len(args) == 1:
                print("tickresolution: %.1f" % self.tickresolution)
                return
            self.tickresolution = float(args[1])
        elif args[0] == "legend":
            self.cmd_legend(args[1:])
        else:
            # start a new graph
            self.graphs.append(Graph(self, args[:]))

    def cmd_legend(self, args):
        '''setup legend for graphs'''
        if len(args) == 0:
            for leg in self.legend.keys():
                print("%s -> %s" % (leg, self.legend[leg]))
        elif len(args) == 1:
            leg = args[0]
            if leg in self.legend:
                print("Removing legend %s" % leg)
                self.legend.pop(leg)
        elif len(args) >= 2:
            leg = args[0]
            leg2 = args[1]
            print("Adding legend %s -> %s" % (leg, leg2))
            self.legend[leg] = leg2

    def unload(self):
        '''unload module'''
        for g in self.graphs:
            g.close()
        self.graphs = []

    def mavlink_packet(self, msg):
        '''handle an incoming mavlink packet'''

        # check for any closed graphs
        for i in range(len(self.graphs) - 1, -1, -1):
            if not self.graphs[i].is_alive():
                self.graphs[i].close()
                self.graphs.pop(i)

        # add data to the rest
        for g in self.graphs:
            g.add_mavlink_packet(msg)


def init(mpstate):
    '''initialise module'''
    return GraphModule(mpstate)

class Graph():
    '''a graph instance'''
    def __init__(self, state, fields):
        self.fields = fields[:]
        self.field_types = []
        self.msg_types = set()
        self.state = state

        for i in range(len(self.fields)):
            # special handling for NAMED_VALUE_FLOAT
            m = re.match("^NAMED_VALUE_FLOAT\[([A-Z0-9_]+)\]\.(.*)$", self.fields[i])
            if m:
                self.fields[i] = 'NAMED_VALUE_FLOAT["%s"].%s' % (m.group(1), m.group(2))

        re_caps = re.compile('[A-Z_][A-Z0-9_]+')
        for f in self.fields:
            caps = set(re.findall(re_caps, f))
            self.msg_types = self.msg_types.union(caps)
            self.field_types.append(caps)
        print("Adding graph: %s" % self.fields)

        fields = [ self.pretty_print_fieldname(x) for x in fields ]

        self.values = [None] * len(self.fields)
        self.livegraph = live_graph.LiveGraph(fields,
                                              timespan=state.timespan,
                                              tickresolution=state.tickresolution,
                                              title=self.fields[0])

    def pretty_print_fieldname(self, fieldname):
        if fieldname in self.state.legend:
            return self.state.legend[fieldname]
        return fieldname

    def is_alive(self):
        '''check if this graph is still alive'''
        if self.livegraph:
            return self.livegraph.is_alive()
        return False

    def close(self):
        '''close this graph'''
        if self.livegraph:
            self.livegraph.close()
        self.livegraph = None

    def add_mavlink_packet(self, msg):
        '''add data to the graph'''
        mtype = msg.get_type()
        if mtype not in self.msg_types:
            return
        have_value = False
        for i in range(len(self.fields)):
            if mtype not in self.field_types[i]:
                continue
            f = self.fields[i]
            self.values[i] = mavutil.evaluate_expression(f, self.state.master.messages)
            if self.values[i] is not None:
                have_value = True
        if have_value and self.livegraph is not None:
            self.livegraph.add_values(self.values)



#!/usr/bin/env python
'''
Tornado server for Cesium map module
Samuel Dudley
Jan 2016
'''
import tornado.ioloop
import tornado.web
import tornado.websocket
import tornado.httpserver
import logging
                
import os, json, sys, select, signal, threading
try:
    import Queue as queue
except ImportError:
    import queue

lock = threading.Lock()
live_web_sockets = set()

try: # try to use pkg_resources to allow for zipped python eggs
    import pkg_resources
    APP_ROOT = pkg_resources.resource_filename('MAVProxy.modules.mavproxy_cesium','app')
    APP_STATIC = pkg_resources.resource_filename('MAVProxy.modules.mavproxy_cesium.app','static')
    APP_TEMPLATES = pkg_resources.resource_filename('MAVProxy.modules.mavproxy_cesium.app','templates')
except: # otherwise fall back to the standard file system
    APP_ROOT = os.path.dirname(os.path.abspath(__file__))
    APP_STATIC = os.path.join(APP_ROOT, 'static')
    APP_TEMPLATES = os.path.join(APP_ROOT, 'templates')

class MainHandler(tornado.web.RequestHandler):
    def initialize(self, configuration, callback):
        self.configuration = configuration
        
    def get(self):
        self.render("index.html", bing_api_key=self.configuration.BING_API_KEY, websocket=self.configuration.WEBSOCKET, markers=False,
                     app_prefix = self.configuration.APP_PREFIX)
        
class ContextHandler(tornado.web.RequestHandler):
    def initialize(self, configuration, callback):
        self.configuration = configuration
        
    def post(self):
        markers = [self.get_argument("markers", default=False, strip=True).lstrip('"').rstrip('"')]
        if 'null' in markers:
            markers = False
        self.render("context_menu.html", markers=markers)

class DefaultWebSocket(tornado.websocket.WebSocketHandler):
    def initialize(self, configuration, callback):
        self.configuration = configuration
        self.callback = callback
    
    def check_origin(self, origin):
        return True
        
    def open(self):
        if self.configuration.APP_DEBUG:
            print("websocket opened!")
        self.set_nodelay(True)
        
        lock.acquire()
        live_web_sockets.add(self)
        lock.release()
     
    def on_message(self, message):
        if self.configuration.APP_DEBUG:
            print("received websocket message: {0}".format(message))
        message = json.loads(message)
        if self.callback:
            self.callback(message) # this sends it to the module.send_out_queue_data for further processing.
        else:
            print("no callback for message: {0}".format(message))
        print(dir(self))

    def on_close(self):
        if self.configuration.APP_DEBUG:
            print("websocket closed")
        del self

class Application(tornado.web.Application):
    def __init__(self, config, module_callback):
        args = dict(configuration=config, callback=module_callback)
        handlers = [
            (r"/"+config.APP_PREFIX, MainHandler, args),
            (r"/"+config.APP_PREFIX+"context/", ContextHandler, args),
            (r"/"+config.APP_PREFIX+"websocket/", DefaultWebSocket, args),
        ]

        settings = dict(
            cookie_secret = config.APP_SECRET_KEY,
            template_path = APP_TEMPLATES,
            static_path = APP_STATIC,
            static_url_prefix = "/"+config.APP_PREFIX+"static/",
            xsrf_cookies = False,
        )
        super(Application, self).__init__(handlers, **settings)

def start_app(config, module_callback):
    logging.getLogger("tornado").setLevel(logging.WARNING)
    application = Application(config, module_callback)
    server = tornado.httpserver.HTTPServer(application)
    server.listen(port = int(config.SERVER_PORT), address = str(config.SERVER_INTERFACE))
    if config.APP_DEBUG:
        print("Starting Tornado server: {0}".format(config.SERVER_INTERFACE+":"+config.SERVER_PORT+"/"+config.APP_PREFIX))
    return server

def close_all_websockets():
    removable = set()
    lock.acquire()
    for ws in live_web_sockets:
        removable.add(ws)
    for ws in removable:
        live_web_sockets.remove(ws)
    lock.release()
            
def stop_tornado(config):
    close_all_websockets()
    ioloop = tornado.ioloop.IOLoop.current()
    ioloop.add_callback(ioloop.stop)
    if config.APP_DEBUG:
        print("Asked Tornado to exit")

def websocket_send_message(message):
    removable = set()
    lock.acquire()
    for ws in live_web_sockets:
        if not ws.ws_connection or not ws.ws_connection.stream.socket:
            removable.add(ws)
        else:
            ws.write_message(message)
    lock.release()
    
    lock.acquire()
    for ws in removable:
        live_web_sockets.remove(ws)
    lock.release()

def main(config, module_callback):
    server = start_app(config=config, module_callback=module_callback)
    tornado.ioloop.IOLoop.current().start()
    if config.APP_DEBUG:
        print("Tornado finished")
    server.stop()
    
class Connection(object):
    def __init__(self, connection):
        self.control_connection = connection # a MAVLink connection
        self.control_link = self.control_connection.mav
        self.control_link.srcSystem = 11
        self.control_link.srcComponent = 220 
        
    def set_component(self, val):
        self.control_link.srcComponent = val
    
    def set_system(self, val):
        self.control_link.srcSystem = val
     
class module(object):
    def __init__(self, optsargs):
        self.exit = False
        (self.opts, self.args) = optsargs
        self.server_thread = None
        self.config = Configuration(self.opts.configuration)
        signal.signal(signal.SIGINT, self.exit_gracefully)
        signal.signal(signal.SIGTERM, self.exit_gracefully)
        self.message_queue = queue.Queue(maxsize=10) # limit queue object count to 10
        self.pos_target = {}
        self.data_stream = ['NAV_CONTROLLER_OUTPUT', 'VFR_HUD',
                            'ATTITUDE', 'GLOBAL_POSITION_INT',
                            'SYS_STATUS', 'MISSION_CURRENT',
                            'STATUSTEXT', 'FENCE_STATUS', 'WIND']
        try:
            self.connection = Connection(mavutil.mavlink_connection(self.opts.connection))
        except Exception as err:
            print("Failed to connect to %s : %s" %(self.opts.connection,err))
            sys.exit(1)
        self.server_thread = threading.Thread(target=main, args = (self.config, self.callback,))
        self.server_thread.daemon = True
        self.server_thread.start()
        self.main_loop()
    
    def callback(self, data):
        '''callback for data coming in from a websocket'''
        try:
            self.message_queue.put_nowait(data)
        except queue.Full:
            print ('Queue full, client data is unable to be enqueued')
        
    def send_data(self, data, target = None):
        '''push json data to the browser via a websocket'''
        payload = json.dumps(data).encode('utf8')
        websocket_send_message(payload)
        # TODO: direct messages to individual websockets, e.g. new connections
        
    def drain_message_queue(self):
        '''unload data that has been placed on the message queue by the client'''
        while not self.message_queue.empty():
            try:
                data = self.message_queue.get_nowait()
            except queue.Empty:
                return
            else:
                # TODO: handle the user feedback
                pass
            
    def process_connection_in(self):
        '''receive MAVLink messages'''
        try:
            inputready,outputready,exceptready = select.select([self.connection.control_connection.port],[],[],0.1)
            # block for 0.1 sec if there is nothing on the connection
            # otherwise we just dive right in...
            for s in inputready: 
                self.connection.control_connection.recv_msg()
            # mavlink buffer is never getting cleared
            # force clear the buffer to avoid memory leak
            if self.connection.control_connection.mav.buf_len() == 0 and self.connection.control_connection.mav.buf_index != 0:
                self.connection.control_connection.mav.buf = bytearray()
                self.connection.control_connection.mav.buf_index = 0
        except select.error:
            pass
            
    def handle_msg(self, con, msg):
        '''callback for received MAVLink messages''' 
        if msg.get_type() == 'POSITION_TARGET_GLOBAL_INT':
            msg_dict = msg.to_dict()
            self.pos_target['lat']= msg_dict['lat_int']
            self.pos_target['lon'] = msg_dict['lon_int']
            self.pos_target['alt_wgs84'] = msg_dict['alt']
         
            self.send_data({"pos_target_data":self.pos_target})
            
        elif msg.get_type() in self.data_stream:
            msg_dict = msg.to_dict()
            msg_dict['timestamp'] = msg._timestamp
            self.send_data({'mav_data':msg_dict})
        else:
            # message type not handleded 
            pass
                    
    def main_loop(self):
        '''main loop of the module'''
        self.connection.control_connection.message_hooks.append(self.handle_msg)
        self.connection.control_link.request_data_stream_send(1, 1,
                                                mavutil.mavlink.MAV_DATA_STREAM_ALL,
                                                self.opts.stream_rate, 1)
        while not self.exit:
            self.process_connection_in() # any down time (max 0.1 sec) occurs here
            self.drain_message_queue()
        print('Module finished')
    
    def exit_gracefully(self, signum, frame):
        self.exit = True
        if self.server_thread:
            # attempt to shutdown the tornado server
            stop_tornado(self.config)
            self.server_thread.join(timeout=10)
        
if __name__ == '__main__':
    # we are running outside MAVProxy in stand alone mode
    from optparse import OptionParser
    from config import Configuration
    
    parser = OptionParser('cesium_web_server.py [options]')
    
    parser.add_option("--connection", dest="connection", type='str',
                      help="MAVLink computer connection", default="tcp:127.0.0.1:5763")
    parser.add_option("--dialect", dest="dialect", help="MAVLink dialect", default="ardupilotmega")
    parser.add_option("--stream-rate", dest="stream_rate", help="requested MAVLink stream rate from AP", type='int', default=10)
    parser.add_option("--configuration", dest="configuration", type='str',
                      help="path to MAVCesium configuration file", default=None)
    optsargs = parser.parse_args()
    (opts,args) = optsargs
    
    os.environ['MAVLINK20'] = '1' # force MAVLink v2 for the moment
    from pymavlink import mavutil
    mavutil.set_dialect(opts.dialect)
    module(optsargs)
    
    

    
     
    